@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -23,6 +23,7 @@
 #include <bitset>
 #include <vector>
 #include <string>
+#include <map>
 
 typedef std::vector<CItemField> UnknownFields;
 typedef UnknownFields::const_iterator UnknownFieldsConstIter;
@@ -46,6 +47,7 @@ typedef UnknownFields::const_iterator UnknownFieldsConstIter;
 */
 
 class BlowFish;
+class PWSfile;
 
 struct DisplayInfoBase
 {
@@ -61,15 +63,39 @@ public:
   // field types, per formatV{2,3}.txt. Any value > 0xff is internal only!
   enum FieldType {
     START = 0x00, GROUPTITLE = 0x00 /* reusing depreciated NAME for Group.Title combination */,
-    NAME = 0x00, UUID = 0x01, GROUP = 0x02, TITLE = 0x03, USER = 0x04, NOTES = 0x05,
-    PASSWORD = 0x06, CTIME = 0x07, PMTIME = 0x08, ATIME = 0x09, XTIME = 0x0a,
-    RESERVED = 0x0b /* cannot use */, RMTIME = 0x0c, URL = 0x0d, AUTOTYPE = 0x0e,
-    PWHIST = 0x0f, POLICY = 0x10, XTIME_INT = 0x11, RUNCMD = 0x12, DCA = 0x13,
-    EMAIL = 0x14, PROTECTED = 0x15, SYMBOLS = 0x16,
+    NAME = 0x00,
+    UUID = 0x01,
+    GROUP = 0x02,
+    TITLE = 0x03,
+    USER = 0x04,
+    NOTES = 0x05,
+    PASSWORD = 0x06,
+    CTIME = 0x07,  // Entry 'C'reation time
+    PMTIME = 0x08, // last 'P'assword 'M'odification time
+    ATIME = 0x09,  // last 'A'ccess time
+    XTIME = 0x0a,  // password e'X'piry time
+    RESERVED = 0x0b /* cannot use */,
+    RMTIME = 0x0c, // last 'R'ecord 'M'odification time
+    URL = 0x0d, AUTOTYPE = 0x0e,
+    PWHIST = 0x0f,
+    POLICY = 0x10, // string encoding of item-specific password policy
+    XTIME_INT = 0x11,
+    RUNCMD = 0x12,
+    DCA = 0x13,    // doubleclick action (enum)
+    EMAIL = 0x14,
+    PROTECTED = 0x15,
+    SYMBOLS = 0x16,    // string of item-specific password symbols
+    SHIFTDCA = 0x17,   // shift-doubleclick action (enum)
+    POLICYNAME = 0x18, // named non-default password policy for item
     LAST,        // Start of unknown fields!
     END = 0xff,
     // Internal fields only - used in filters
-    ENTRYSIZE = 0x100, ENTRYTYPE = 0x101, ENTRYSTATUS  = 0x102, UNKNOWNFIELDS = 0x103};
+    ENTRYSIZE = 0x100, ENTRYTYPE = 0x101, ENTRYSTATUS  = 0x102, PASSWORDLEN = 0x103,
+    // 'UNKNOWNFIELDS' should be last
+    UNKNOWNFIELDS = 0x104};
+
+  // Password Policy stuff: Either PWPolicy (+ optionally symbols) is not empty
+  // or PolicyName is not empty. Both cannot be set. All can be empty.
 
   // SubGroup Object - same as FieldType
 
@@ -93,6 +119,14 @@ public:
                     ES_DELETED      =  4,  // Deleted  but not yet removed from disk copy
                     ES_LAST};
 
+  // Flags if error found during validate of the entry
+  enum  {VF_OK              =  0,
+         VF_BAD_UUID        =  1,
+         VF_EMPTY_TITLE     =  2,
+         VF_EMPTY_PASSWORD  =  4,
+         VF_NOT_UNIQUE_GTU  =  8,
+         VF_BAD_PSWDHISTORY = 16};
+
   // a bitset for indicating a subset of an item's fields: 
   typedef std::bitset<LAST> FieldBits;
 
@@ -102,52 +136,55 @@ public:
 
   //Construction
   CItemData();
-
   CItemData(const CItemData& stuffhere);
 
   ~CItemData();
 
+  int Read(PWSfile *in);
+  int Write(PWSfile *out) const;
+
   // Convenience: Get the name associated with FieldType
   static stringT FieldName(FieldType ft);
   // Convenience: Get the untranslated (English) name of a FieldType
   static stringT EngFieldName(FieldType ft);
 
   //Data retrieval
-  StringX GetName() const; // V17 - deprecated - replaced by GetTitle & GetUser
-  StringX GetTitle() const; // V20
-  StringX GetUser() const; // V20
-  StringX GetPassword() const;
+  StringX GetName() const {return GetField(NAME);} // V17 - deprecated: replaced by GetTitle & GetUser
+  StringX GetTitle() const {return GetField(TITLE);} // V20
+  StringX GetUser() const  {return GetField(USER);}  // V20
+  StringX GetPassword() const {return GetField(PASSWORD);}
+  size_t GetPasswordLength() const {return GetField(PASSWORD).length();}
   StringX GetNotes(TCHAR delimiter = 0) const;
   void GetUUID(uuid_array_t &) const; // V20
   const pws_os::CUUID GetUUID() const; // V20 - see comment in .cpp re return type
-  StringX GetGroup() const; // V20
-  StringX GetURL() const; // V30
-  StringX GetAutoType() const; // V30
-  StringX GetATime() const {return GetTime(ATIME, TMC_ASC_UNKNOWN);}  // V30
-  StringX GetCTime() const {return GetTime(CTIME, TMC_ASC_UNKNOWN);}  // V30
-  StringX GetXTime() const {return GetTime(XTIME, TMC_ASC_UNKNOWN);}  // V30
-  StringX GetPMTime() const {return GetTime(PMTIME, TMC_ASC_UNKNOWN);}  // V30
-  StringX GetRMTime() const {return GetTime(RMTIME, TMC_ASC_UNKNOWN);}  // V30
-  StringX GetATimeL() const {return GetTime(ATIME, TMC_LOCALE);}  // V30
-  StringX GetCTimeL() const {return GetTime(CTIME, TMC_LOCALE);}  // V30
-  StringX GetXTimeL() const {return GetTime(XTIME, TMC_LOCALE);}  // V30
-  StringX GetPMTimeL() const {return GetTime(PMTIME, TMC_LOCALE);}  // V30
-  StringX GetRMTimeL() const {return GetTime(RMTIME, TMC_LOCALE);}  // V30
-  StringX GetATimeN() const {return GetTime(ATIME, TMC_ASC_NULL);}  // V30
-  StringX GetCTimeN() const {return GetTime(CTIME, TMC_ASC_NULL);}  // V30
-  StringX GetXTimeN() const {return GetTime(XTIME, TMC_ASC_NULL);}  // V30
-  StringX GetPMTimeN() const {return GetTime(PMTIME, TMC_ASC_NULL);}  // V30
-  StringX GetRMTimeN() const {return GetTime(RMTIME, TMC_ASC_NULL);}  // V30
-  StringX GetATimeExp() const {return GetTime(ATIME, TMC_EXPORT_IMPORT);}  // V30
-  StringX GetCTimeExp() const {return GetTime(CTIME, TMC_EXPORT_IMPORT);}  // V30
-  StringX GetXTimeExp() const {return GetTime(XTIME, TMC_EXPORT_IMPORT);}  // V30
-  StringX GetPMTimeExp() const {return GetTime(PMTIME, TMC_EXPORT_IMPORT);}  // V30
-  StringX GetRMTimeExp() const {return GetTime(RMTIME, TMC_EXPORT_IMPORT);}  // V30
-  StringX GetATimeXML() const {return GetTime(ATIME, TMC_XML);}  // V30
-  StringX GetCTimeXML() const {return GetTime(CTIME, TMC_XML);}  // V30
-  StringX GetXTimeXML() const {return GetTime(XTIME, TMC_XML);}  // V30
-  StringX GetPMTimeXML() const {return GetTime(PMTIME, TMC_XML);}  // V30
-  StringX GetRMTimeXML() const {return GetTime(RMTIME, TMC_XML);}  // V30
+  StringX GetGroup() const {return GetField(GROUP);} // V20
+  StringX GetURL() const {return GetField(URL);} // V30
+  StringX GetAutoType() const {return GetField(AUTOTYPE);} // V30
+  StringX GetATime() const {return GetTime(ATIME, PWSUtil::TMC_ASC_UNKNOWN);}  // V30
+  StringX GetCTime() const {return GetTime(CTIME, PWSUtil::TMC_ASC_UNKNOWN);}  // V30
+  StringX GetXTime() const {return GetTime(XTIME, PWSUtil::TMC_ASC_UNKNOWN);}  // V30
+  StringX GetPMTime() const {return GetTime(PMTIME, PWSUtil::TMC_ASC_UNKNOWN);}  // V30
+  StringX GetRMTime() const {return GetTime(RMTIME, PWSUtil::TMC_ASC_UNKNOWN);}  // V30
+  StringX GetATimeL() const {return GetTime(ATIME, PWSUtil::TMC_LOCALE);}  // V30
+  StringX GetCTimeL() const {return GetTime(CTIME, PWSUtil::TMC_LOCALE);}  // V30
+  StringX GetXTimeL() const {return GetTime(XTIME, PWSUtil::TMC_LOCALE_DATE_ONLY);}  // V30
+  StringX GetPMTimeL() const {return GetTime(PMTIME, PWSUtil::TMC_LOCALE);}  // V30
+  StringX GetRMTimeL() const {return GetTime(RMTIME, PWSUtil::TMC_LOCALE);}  // V30
+  StringX GetATimeN() const {return GetTime(ATIME, PWSUtil::TMC_ASC_NULL);}  // V30
+  StringX GetCTimeN() const {return GetTime(CTIME, PWSUtil::TMC_ASC_NULL);}  // V30
+  StringX GetXTimeN() const {return GetTime(XTIME, PWSUtil::TMC_ASC_NULL);}  // V30
+  StringX GetPMTimeN() const {return GetTime(PMTIME, PWSUtil::TMC_ASC_NULL);}  // V30
+  StringX GetRMTimeN() const {return GetTime(RMTIME, PWSUtil::TMC_ASC_NULL);}  // V30
+  StringX GetATimeExp() const {return GetTime(ATIME, PWSUtil::TMC_EXPORT_IMPORT);}  // V30
+  StringX GetCTimeExp() const {return GetTime(CTIME, PWSUtil::TMC_EXPORT_IMPORT);}  // V30
+  StringX GetXTimeExp() const {return GetTime(XTIME, PWSUtil::TMC_EXPORT_IMPORT);}  // V30
+  StringX GetPMTimeExp() const {return GetTime(PMTIME, PWSUtil::TMC_EXPORT_IMPORT);}  // V30
+  StringX GetRMTimeExp() const {return GetTime(RMTIME, PWSUtil::TMC_EXPORT_IMPORT);}  // V30
+  StringX GetATimeXML() const {return GetTime(ATIME, PWSUtil::TMC_XML);}  // V30
+  StringX GetCTimeXML() const {return GetTime(CTIME, PWSUtil::TMC_XML);}  // V30
+  StringX GetXTimeXML() const {return GetTime(XTIME, PWSUtil::TMC_XML);}  // V30
+  StringX GetPMTimeXML() const {return GetTime(PMTIME, PWSUtil::TMC_XML);}  // V30
+  StringX GetRMTimeXML() const {return GetTime(RMTIME, PWSUtil::TMC_XML);}  // V30
   //  These populate the time structure instead of giving a character string
   void GetATime(time_t &t) const {GetTime(ATIME, t);}  // V30
   void GetCTime(time_t &t) const {GetTime(CTIME, t);}  // V30
@@ -158,17 +195,20 @@ public:
   StringX GetXTimeInt() const; // V30
   StringX GetPWHistory() const;  // V30
   void GetPWPolicy(PWPolicy &pwp) const;
-  StringX GetPWPolicy() const;
-  StringX GetRunCommand() const;
-  void GetDCA(short &iDCA)  const;
-  StringX GetDCA() const;
-  StringX GetEmail() const;
+  StringX GetPWPolicy() const {return GetField(POLICY);}
+  StringX GetRunCommand() const {return GetField(RUNCMD);}
+  void GetDCA(short &iDCA, const bool bShift = false) const;
+  StringX GetDCA(const bool bShift = false) const;
+  void GetShiftDCA(short &iDCA) const {GetDCA(iDCA, true);}
+  StringX GetShiftDCA() const {return GetDCA(true);}
+  StringX GetEmail() const {return GetField(EMAIL);}
   StringX GetProtected() const;
   void GetProtected(unsigned char &ucprotected) const;
   bool IsProtected() const;
-  StringX GetSymbols() const;
+  StringX GetSymbols() const    {return GetField(SYMBOLS);}
+  StringX GetPolicyName() const {return GetField(POLICYNAME);}
 
-  StringX GetFieldValue(const FieldType &ft) const;
+  StringX GetFieldValue(FieldType ft) const;
 
   // GetPlaintext returns all fields separated by separator, if delimiter is != 0, then
   // it's used for multi-line notes and to replace '.' within the Title field.
@@ -176,20 +216,11 @@ public:
                        const TCHAR &delimiter, const CItemData *pcibase) const;
   std::string GetXML(unsigned id, const FieldBits &bsExport, TCHAR m_delimiter,
                      const CItemData *pcibase, bool bforce_normal_entry) const;
-  void GetUnknownField(unsigned char &type, size_t &length,
-                       unsigned char * &pdata,
-                       const unsigned int &num) const;
-  void GetUnknownField(unsigned char &type, size_t &length,
-                       unsigned char * &pdata,
-                       const UnknownFieldsConstIter &iter) const;
-  void SetUnknownField(const unsigned char &type, const size_t &length,
-                       const unsigned char * &ufield);
-  size_t NumberUnknownFields() const
-  {return m_URFL.size();}
-  void ClearUnknownFields()
-  {return m_URFL.clear();}
-  UnknownFieldsConstIter GetURFIterBegin() const {return m_URFL.begin();}
-  UnknownFieldsConstIter GetURFIterEnd() const {return m_URFL.end();}
+
+  void SetUnknownField(unsigned char type, size_t length,
+                       const unsigned char *ufield);
+  size_t NumberUnknownFields() const {return m_URFL.size();}
+  void ClearUnknownFields() {return m_URFL.clear();}
 
   void CreateUUID(); // V20 - generate UUID for new item
   void SetName(const StringX &name,
@@ -225,23 +256,29 @@ public:
   void SetPWPolicy(const PWPolicy &pwp);
   bool SetPWPolicy(const stringT &cs_pwp);
   void SetRunCommand(const StringX &cs_RunCommand);
-  void SetDCA(const short &iDCA);
-  bool SetDCA(const stringT &cs_DCA);
+  void SetDCA(const short &iDCA, const bool bShift = false);
+  bool SetDCA(const stringT &cs_DCA, const bool bShift = false);
+  void SetShiftDCA(const short &iDCA) {SetDCA(iDCA, true);}
+  bool SetShiftDCA(const stringT &cs_DCA) {return SetDCA(cs_DCA, true);}
   void SetEmail(const StringX &sx_email);
-  void SetProtected(const bool &bOnOff);
+  void SetProtected(bool bOnOff);
   void SetSymbols(const StringX &sx_symbols);
+  void SetPolicyName(const StringX &sx_PolicyName);
 
-  void SetFieldValue(const FieldType &ft, const StringX &value);
+  void SetFieldValue(FieldType ft, const StringX &value);
 
   CItemData& operator=(const CItemData& second);
   // Following used by display methods - we just keep it handy
   DisplayInfoBase *GetDisplayInfo() const {return m_display_info;}
   void SetDisplayInfo(DisplayInfoBase *di) {delete m_display_info; m_display_info = di;}
   void Clear();
+  void ClearField(FieldType ft) {m_fields.erase(ft);}
+
+
   // check record for mandatory fields, silently fix if missing
-  int ValidateUUID(const unsigned short &nMajor, const unsigned short &nMinor,
-                   uuid_array_t &uuid_array);
+  bool ValidateEntry(int &flags);
   bool ValidatePWHistory(); // return true if OK, false if there's a problem
+
   bool IsExpired() const;
   bool WillExpire(const int numdays) const;
 
@@ -252,43 +289,44 @@ public:
                int iFunction) const;  // integer values
   bool Matches(time_t time1, time_t time2, int iObject,
                int iFunction) const;  // time values
-  bool Matches(short dca, int iFunction) const;  // DCA values
+  bool Matches(short dca, int iFunction, const bool bShift = false) const;  // DCA values
   bool Matches(EntryType etype, int iFunction) const;  // Entrytype values
   bool Matches(EntryStatus estatus, int iFunction) const;  // Entrystatus values
 
-  bool IsGroupEmpty() const {return m_Group.IsEmpty();}
-  bool IsUserEmpty() const {return m_User.IsEmpty();}
-  bool IsNotesEmpty() const {return m_Notes.IsEmpty();}
-  bool IsURLEmpty() const {return m_URL.IsEmpty();}
-  bool IsRunCommandEmpty() const {return m_RunCommand.IsEmpty();}
-  bool IsEmailEmpty() const {return m_email.IsEmpty();}
-
-  bool IsGroupSet() const                     { return !m_Group.IsEmpty();        }
-  bool IsUserSet() const                      { return !m_User.IsEmpty();         }
-  bool IsNotesSet() const                     { return !m_Notes.IsEmpty();        }
-  bool IsURLSet() const                       { return !m_URL.IsEmpty();          }
-  bool IsRunCommandSet() const                { return !m_RunCommand.IsEmpty();   }
-  bool IsEmailSet() const                     { return !m_email.IsEmpty();        }
-  bool IsUUIDSet() const                      { return !m_UUID.IsEmpty();         }
-  bool IsTitleSet() const                     { return !m_Title.IsEmpty();        }
-  bool IsPasswordSet() const                  { return !m_Password.IsEmpty();     }
-  bool IsCreationTimeSet() const              { return !m_tttCTime.IsEmpty();     }
-  bool IsModificationTimeSet() const          { return !m_tttPMTime.IsEmpty();    }
-  bool IsLastAccessTimeSet() const            { return !m_tttATime.IsEmpty();     }
-  bool IsExpiryDateSet() const                { return !m_tttXTime.IsEmpty();     }
-  bool IsRecordModificationTimeSet() const    { return !m_tttRMTime.IsEmpty();    }
-  bool IsAutoTypeSet() const                  { return !m_AutoType.IsEmpty();     }
-  bool IsPasswordHistorySet() const           { return !m_PWHistory.IsEmpty();    }
-  bool IsPasswordPolicySet() const            { return !m_PWPolicy.IsEmpty();     }
-  bool IsPasswordExpiryIntervalSet() const    { return !m_XTimeInterval.IsEmpty();}
-  bool IsDCASet() const                       { return !m_DCA.IsEmpty();          }
-  bool IsProtectionSet() const                { return !m_protected.IsEmpty();    }
-  bool IsSymbolsSet() const                   { return !m_symbols.IsEmpty();      }
+  bool IsGroupSet() const                  { return IsFieldSet(GROUP);     }
+  bool IsUserSet() const                   { return IsFieldSet(USER);      }
+  bool IsNotesSet() const                  { return IsFieldSet(NOTES);     }
+  bool IsURLSet() const                    { return IsFieldSet(URL);       }
+  bool IsRunCommandSet() const             { return IsFieldSet(RUNCMD);    }
+  bool IsEmailSet() const                  { return IsFieldSet(EMAIL);     }
+  bool IsUUIDSet() const                   { return IsFieldSet(UUID);      }
+  bool IsTitleSet() const                  { return IsFieldSet(TITLE);     }
+  bool IsPasswordSet() const               { return IsFieldSet(PASSWORD);  }
+  bool IsCreationTimeSet() const           { return IsFieldSet(CTIME);     }
+  bool IsModificationTimeSet() const       { return IsFieldSet(PMTIME);    }
+  bool IsLastAccessTimeSet() const         { return IsFieldSet(ATIME);     }
+  bool IsExpiryDateSet() const             { return IsFieldSet(XTIME);     }
+  bool IsRecordModificationTimeSet() const { return IsFieldSet(RMTIME);    }
+  bool IsAutoTypeSet() const               { return IsFieldSet(AUTOTYPE);  }
+  bool IsPasswordHistorySet() const        { return IsFieldSet(PWHIST);    }
+  bool IsPasswordPolicySet() const         { return IsFieldSet(POLICY);    }
+  bool IsPasswordExpiryIntervalSet() const { return IsFieldSet(XTIME_INT); }
+  bool IsDCASet() const                    { return IsFieldSet(DCA);       }
+  bool IsProtectionSet() const             { return IsFieldSet(PROTECTED); }
+  bool IsSymbolsSet() const                { return IsFieldSet(SYMBOLS);   }
+  bool IsPolicyNameSet() const             { return IsFieldSet(POLICYNAME);}
     
+  bool IsGroupEmpty() const                { return !IsGroupSet();         }
+  bool IsUserEmpty() const                 { return !IsUserSet();          }
+  bool IsNotesEmpty() const                { return !IsNotesSet();         }
+  bool IsURLEmpty() const                  { return !IsURLSet();           }
+  bool IsRunCommandEmpty() const           { return !IsRunCommandSet();    }
+  bool IsEmailEmpty() const                { return !IsEmailSet();         }
+  bool IsPolicyEmpty() const               { return !IsPasswordPolicySet();}
+
   void SerializePlainText(std::vector<char> &v,
                           const CItemData *pcibase = NULL) const;
-  bool DeserializePlainText(const std::vector<char> &v);
-  bool SetField(int type, const unsigned char *data, size_t len);
+  bool DeSerializePlainText(const std::vector<char> &v);
 
   EntryType GetEntryType() const {return m_entrytype;}
 
@@ -307,42 +345,23 @@ public:
   void SetAlias() {m_entrytype = ET_ALIAS;}
   void SetShortcut() {m_entrytype = ET_SHORTCUT;}
 
-  EntryStatus GetStatus() const
-  {return m_entrystatus;}
-  void ClearStatus()
-  {m_entrystatus = ES_CLEAN;}
-  void SetStatus(const EntryStatus es)
-  {m_entrystatus = es;}
+  EntryStatus GetStatus() const {return m_entrystatus;}
+  void ClearStatus() {m_entrystatus = ES_CLEAN;}
+  void SetStatus(const EntryStatus es) {m_entrystatus = es;}
 
   bool IsURLEmail() const
   {return GetURL().find(_T("mailto:")) != StringX::npos;}
 
-  size_t GetSize();
-  void GetSize(size_t &isize) const;
+  size_t GetSize() const;
+  void GetSize(size_t &isize) const {isize = GetSize();}
+
 
 private:
-  CItemField m_Name;
-  CItemField m_Title;
-  CItemField m_User;
-  CItemField m_Password;
-  CItemField m_Notes;
-  CItemField m_UUID;
-  CItemField m_Group;
-  CItemField m_URL;
-  CItemField m_AutoType;
-  CItemField m_tttATime;  // last 'A'ccess time
-  CItemField m_tttCTime;  // 'C'reation time
-  CItemField m_tttXTime;  // password e'X'iry time
-  CItemField m_tttPMTime; // last 'P'assword 'M'odification time
-  CItemField m_tttRMTime; // last 'R'ecord 'M'odification time
-  CItemField m_PWHistory;
-  CItemField m_PWPolicy;
-  CItemField m_XTimeInterval;
-  CItemField m_RunCommand;
-  CItemField m_DCA;
-  CItemField m_email;
-  CItemField m_protected;
-  CItemField m_symbols;
+  typedef std::map<FieldType, CItemField> FieldMap;
+  typedef FieldMap::const_iterator FieldConstIter;
+  typedef FieldMap::iterator FieldIter;
+
+  FieldMap   m_fields;
 
   // Save unknown record fields on read to put back on write unchanged
   UnknownFields m_URFL;
@@ -362,7 +381,7 @@ private:
   // move from pre-2.0 name to post-2.0 title+user
   void SplitName(const StringX &name,
                  StringX &title, StringX &username);
-  StringX GetTime(int whichtime, int result_format) const; // V30
+  StringX GetTime(int whichtime, PWSUtil::TMC result_format) const; // V30
   void GetTime(int whichtime, time_t &t) const; // V30
   void SetTime(const int whichtime); // V30
   void SetTime(const int whichtime, time_t t); // V30
@@ -371,22 +390,31 @@ private:
   // Create local Encryption/Decryption object
   BlowFish *MakeBlowFish(bool noData = false) const;
   // Laziness is a Virtue:
+  StringX GetField(FieldType ft) const;
   StringX GetField(const CItemField &field) const;
   void GetField(const CItemField &field, unsigned char *value,
                 size_t &length) const;
+
+  void SetField(FieldType ft, const StringX &value);
+  void SetField(FieldType ft, const unsigned char *value, size_t length);
+  bool SetField(int type, const unsigned char *data, size_t len);
+
+  bool IsFieldSet(FieldType ft) const {return m_fields.find(ft) != m_fields.end();}
+
+  void UpdatePasswordHistory(); // used by UpdatePassword()
+
   void GetUnknownField(unsigned char &type, size_t &length,
                        unsigned char * &pdata, const CItemField &item) const;
-  void SetField(CItemField &field, const StringX &value);
-  void SetField(CItemField &field, const unsigned char *value,
-                size_t length);
-  void UpdatePasswordHistory(); // used by UpdatePassword()
+  int WriteUnknowns(PWSfile *out) const;
+  size_t WriteIfSet(FieldType ft, PWSfile *out, bool isUTF8) const;
 };
 
 inline bool CItemData::IsTextField(unsigned char t)
 {
-  return !(t == UUID || t == CTIME || t == PMTIME ||
-    t == ATIME || t == XTIME || t == RMTIME || t == XTIME_INT ||
-    t == RESERVED || t == DCA ||
+  return !(t == UUID ||
+    t == CTIME     || t == PMTIME || t == ATIME || t == XTIME || t == RMTIME ||
+    t == XTIME_INT ||
+    t == RESERVED  || t == DCA    || t == SHIFTDCA || t == PROTECTED ||
     t >= LAST);
 }
 #endif /* __ITEMDATA_H */
