@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -74,8 +74,8 @@ namespace PWSMatch {
 
   bool Match(bool bValue, int iFunction);  // bool - if field present or not
 
-  const UINT GetRule(MatchRule rule);
-  const MatchRule GetRule(const StringX &sx_mnemonic);
+  UINT GetRule(MatchRule rule);
+  MatchRule GetRule(const StringX &sx_mnemonic);
   const char *GetRuleString(const MatchRule rule);
   void GetMatchType(MatchType mtype,
                     int fnum1, int fnum2,
