@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,6 +17,7 @@
 
 #include "os/debug.h"
 #include "os/file.h"
+#include "os/logit.h"
 
 #include "XML/XMLDefs.h"  // Required if testing "USE_XML_LIBRARY"
 
@@ -50,6 +51,8 @@ PWSfileV3::~PWSfileV3()
 
 int PWSfileV3::Open(const StringX &passkey)
 {
+  PWS_LOGIT;
+
   int status = SUCCESS;
 
   ASSERT(m_curversion == V30);
@@ -77,6 +80,8 @@ int PWSfileV3::Open(const StringX &passkey)
 
 int PWSfileV3::Close()
 {
+  PWS_LOGIT;
+
   if (m_fd == NULL)
     return SUCCESS; // idempotent
 
@@ -159,6 +164,8 @@ int PWSfileV3::CheckPasskey(const StringX &filename,
                             const StringX &passkey, FILE *a_fd,
                             unsigned char *aPtag, int *nITER)
 {
+  PWS_LOGIT;
+
   FILE *fd = a_fd;
   int retval = SUCCESS;
   SHA256 H;
@@ -193,7 +200,7 @@ int PWSfileV3::CheckPasskey(const StringX &filename,
     unsigned char Ptag[SHA256::HASHLEN];
     if (aPtag == NULL)
       aPtag = Ptag;
-    
+
     StretchKey(salt, sizeof(salt), passkey, N, aPtag);
   }
   unsigned char HPtag[SHA256::HASHLEN];
@@ -233,100 +240,7 @@ int PWSfileV3::WriteRecord(const CItemData &item)
 {
   ASSERT(m_fd != NULL);
   ASSERT(m_curversion == V30);
-  int status = SUCCESS;
-  StringX tmp;
-  uuid_array_t item_uuid;
-  time_t t = 0;
-  int32 i32;
-  short i16;
-
-  item.GetUUID(item_uuid);
-  WriteCBC(CItemData::UUID, item_uuid, sizeof(uuid_array_t));
-  tmp = item.GetGroup();
-  if (!tmp.empty())
-    WriteCBC(CItemData::GROUP, tmp);
-  WriteCBC(CItemData::TITLE, item.GetTitle());
-  WriteCBC(CItemData::USER, item.GetUser());
-  WriteCBC(CItemData::PASSWORD, item.GetPassword());
-  tmp = item.GetNotes();
-  if (!tmp.empty())
-    WriteCBC(CItemData::NOTES, tmp);
-  tmp = item.GetURL();
-  if (!tmp.empty())
-    WriteCBC(CItemData::URL, tmp);
-  tmp = item.GetAutoType();
-  if (!tmp.empty())
-    WriteCBC(CItemData::AUTOTYPE, tmp);
-  item.GetCTime(t);
-  if (t != 0) {
-    i32 = static_cast<int>(t);
-    WriteCBC(CItemData::CTIME, reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
-  }
-  item.GetPMTime(t);
-  if (t != 0) {
-    i32 = static_cast<int>(t);
-    WriteCBC(CItemData::PMTIME, reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
-  }
-  item.GetATime(t);
-  if (t != 0) {
-    i32 = static_cast<int>(t);
-    WriteCBC(CItemData::ATIME, reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
-  }
-  item.GetXTime(t);
-  if (t != 0) {
-    i32 = static_cast<int>(t);
-    WriteCBC(CItemData::XTIME, reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
-  }
-  item.GetXTimeInt(i32);
-  if (i32 > 0 && i32 <= 3650) {
-    WriteCBC(CItemData::XTIME_INT, reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
-  }
-  item.GetRMTime(t);
-  if (t != 0) {
-    i32 = static_cast<int>(t);
-    WriteCBC(CItemData::RMTIME, reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
-  }
-  tmp = item.GetPWPolicy();
-  if (!tmp.empty())
-    WriteCBC(CItemData::POLICY, tmp);
-  tmp = item.GetPWHistory();
-  if (!tmp.empty())
-    WriteCBC(CItemData::PWHIST, tmp);
-  tmp = item.GetRunCommand();
-  if (!tmp.empty())
-    WriteCBC(CItemData::RUNCMD, tmp);
-  item.GetDCA(i16);
-  if (i16 >= PWSprefs::minDCA && i16 <= PWSprefs::maxDCA)
-    WriteCBC(CItemData::DCA, reinterpret_cast<unsigned char *>(&i16), sizeof(short));
-  item.GetShiftDCA(i16);
-  if (i16 >= PWSprefs::minDCA && i16 <= PWSprefs::maxDCA)
-    WriteCBC(CItemData::SHIFTDCA, reinterpret_cast<unsigned char *>(&i16), sizeof(short));
-  tmp = item.GetEmail();
-  if (!tmp.empty())
-    WriteCBC(CItemData::EMAIL, tmp);
-  tmp = item.GetProtected();
-  if (!tmp.empty())
-    WriteCBC(CItemData::PROTECTED, tmp);
-  tmp = item.GetSymbols();
-  if (!tmp.empty())
-    WriteCBC(CItemData::SYMBOLS, tmp);
-
-  UnknownFieldsConstIter vi_IterURFE;
-  for (vi_IterURFE = item.GetURFIterBegin();
-       vi_IterURFE != item.GetURFIterEnd();
-       vi_IterURFE++) {
-    unsigned char type;
-    size_t length = 0;
-    unsigned char *pdata = NULL;
-    item.GetUnknownField(type, length, pdata, vi_IterURFE);
-    WriteCBC(type, pdata, length);
-    trashMemory(pdata, length);
-    delete[] pdata;
-  }
-
-  WriteCBC(CItemData::END, _T(""));
-
-  return status;
+  return item.Write(this);
 }
 
 size_t PWSfileV3::ReadCBC(unsigned char &type, unsigned char* &data,
@@ -345,38 +259,7 @@ int PWSfileV3::ReadRecord(CItemData &item)
 {
   ASSERT(m_fd != NULL);
   ASSERT(m_curversion == V30);
-
-  int status = SUCCESS;
-
-  signed long numread = 0;
-  unsigned char type;
-
-  int emergencyExit = 255; // to avoid endless loop.
-  signed long fieldLen; // <= 0 means end of file reached
-
-  do {
-    unsigned char *utf8 = NULL;
-    size_t utf8Len = 0;
-    fieldLen = static_cast<signed long>(ReadCBC(type, utf8,
-                                                utf8Len));
-
-    if (fieldLen > 0) {
-      numread += fieldLen;
-      if (!item.SetField(type, utf8, utf8Len)) {
-        status = FAILURE;
-        break;
-      }
-    } // if (fieldLen > 0)
-
-    if (utf8 != NULL) {
-      trashMemory(utf8, utf8Len * sizeof(utf8[0]));
-      delete[] utf8; utf8 = NULL; utf8Len = 0;
-    }
-  } while (type != CItemData::END && fieldLen > 0 && --emergencyExit > 0);
-  if (numread > 0)
-    return status;
-  else
-    return END_OF_FILE;
+  return item.Read(this);
 }
 
 void PWSfileV3::StretchKey(const unsigned char *salt, unsigned long saltLen,
@@ -416,7 +299,7 @@ void PWSfileV3::StretchKey(const unsigned char *salt, unsigned long saltLen,
   }
 }
 
-const short VersionNum = 0x0309;
+const short VersionNum = 0x030B;
 
 // Following specific for PWSfileV3::WriteHeader
 #define SAFE_FWRITE(p, sz, cnt, stream) \
@@ -427,6 +310,8 @@ const short VersionNum = 0x0309;
 
 int PWSfileV3::WriteHeader()
 {
+  PWS_LOGIT;
+
   // Following code is divided into {} blocks to
   // prevent "uninitialized" compile errors, as we use
   // goto for error handling
@@ -465,14 +350,14 @@ int PWSfileV3::WriteHeader()
 
   StretchKey(salt, sizeof(salt), m_passkey, NumHashIters, Ptag);
 
-  { 
+  {
     unsigned char HPtag[SHA256::HASHLEN];
     SHA256 H;
     H.Update(Ptag, sizeof(Ptag));
     H.Final(HPtag);
     SAFE_FWRITE(HPtag, 1, sizeof(HPtag), m_fd);
   }
-  { 
+  {
     PWSrand::GetInstance()->GetRandomData(m_key, sizeof(m_key));
     unsigned char B1B2[sizeof(m_key)];
     unsigned char L[32]; // for HMAC
@@ -489,7 +374,7 @@ int PWSfileV3::WriteHeader()
     SAFE_FWRITE(B3B4, 1, sizeof(B3B4), m_fd);
     m_hmac.Init(L, sizeof(L));
   }
-  { 
+  {
     // See discussion on Salt to understand why we hash
     // random data instead of writing it directly
     unsigned char ip_rand[SHA256::HASHLEN];
@@ -594,30 +479,65 @@ int PWSfileV3::WriteHeader()
     oss << setw(2) << setfill('0') << hex << num;
     UUIDListIter iter = m_hdr.m_RUEList.begin();
     // Only save up to max as defined by FormatV3.
-    for (size_t n = 0; n < num; n++) {
+    for (size_t n = 0; n < num; n++, iter++) {
       const uuid_array_t *rep = iter->GetARep();
       for (size_t i = 0; i < sizeof(uuid_array_t); i++) {
         oss << setw(2) << setfill('0') << hex <<  static_cast<unsigned int>((*rep)[i]);
       }
-      iter++;
     }
 
-    numWritten = WriteCBC(HDR_RUE, 
+    numWritten = WriteCBC(HDR_RUE,
                           reinterpret_cast<const unsigned char *>(oss.str().c_str()),
                           oss.str().length());
     if (numWritten <= 0) { status = FAILURE; goto end; }
   }
 
-  if (!m_UHFL.empty()) {
-    UnknownFieldList::iterator vi_IterUHFE;
-    for (vi_IterUHFE = m_UHFL.begin();
-         vi_IterUHFE != m_UHFL.end();
-         vi_IterUHFE++) {
-      UnknownFieldEntry &unkhfe = *vi_IterUHFE;
-      numWritten = WriteCBC(unkhfe.uc_Type,
-                            unkhfe.uc_pUField, static_cast<unsigned int>(unkhfe.st_length));
-      if (numWritten <= 0) { status = FAILURE; goto end; }
+  // Named Policies
+  if (!m_MapPSWDPLC.empty()) {
+    oStringXStream oss;
+    oss.fill(charT('0'));
+
+    size_t num = m_MapPSWDPLC.size();
+    if (num > 255)
+      num = 255;  // Do not exceed 2 hex character length field
+
+    oss << setw(2) << hex << num;
+    PSWDPolicyMapIter iter = m_MapPSWDPLC.begin();
+    for (size_t n = 0; n < num; n++, iter++) {
+      // The Policy name is limited to 255 characters.
+      // This should have been prevented by the GUI.
+      // If not, don't write it out as it may cause issues
+      if (iter->first.length() > 255)
+        continue;
+
+      oss << setw(2) << hex << iter->first.length();
+      oss << iter->first.c_str();
+      StringX strpwp(iter->second);
+      oss << strpwp.c_str();
+      if (iter->second.symbols.empty()) {
+        oss << _T("00");
+      } else {
+        oss << setw(2) << hex << iter->second.symbols.length();
+        oss << iter->second.symbols.c_str();
+      }
     }
+
+    numWritten = WriteCBC(HDR_PSWDPOLICIES, StringX(oss.str().c_str()));
+    if (numWritten <= 0) { status = FAILURE; goto end; }
+  }
+
+  // Empty Groups
+  for (size_t n = 0; n < m_vEmptyGroups.size(); n++) {
+    numWritten = WriteCBC(HDR_EMPTYGROUP, m_vEmptyGroups[n]);
+    if (numWritten <= 0) { status = FAILURE; goto end; }
+  }
+
+  for (UnknownFieldList::iterator vi_IterUHFE = m_UHFL.begin();
+       vi_IterUHFE != m_UHFL.end(); vi_IterUHFE++) {
+    UnknownFieldEntry &unkhfe = *vi_IterUHFE;
+    numWritten = WriteCBC(unkhfe.uc_Type,
+                          unkhfe.uc_pUField, static_cast<unsigned int>(unkhfe.st_length));
+    if (numWritten <= 0) { status = FAILURE; goto end; }
   }
 
   // Write zero-length end-of-record type item
@@ -630,6 +550,8 @@ int PWSfileV3::WriteHeader()
 
 int PWSfileV3::ReadHeader()
 {
+  PWS_LOGIT;
+
   unsigned char Ptag[SHA256::HASHLEN];
   int status = CheckPasskey(m_filename, m_passkey, m_fd,
     Ptag, &m_hdr.m_nITER);
@@ -673,7 +595,7 @@ int PWSfileV3::ReadHeader()
     switch (fieldType) {
       case HDR_VERSION: /* version */
         // in Beta, VersionNum was an int (4 bytes) instead of short (2)
-        // This hack keeps bwd compatability.
+        // This hack keeps bwd compatibility.
         if (utf8Len != sizeof(VersionNum) &&
             utf8Len != sizeof(int32)) {
           delete[] utf8;
@@ -756,7 +678,7 @@ int PWSfileV3::ReadHeader()
             int ulen = 0;
             is >> hex >> ulen;
             StringX uh = text.substr(4);
-            m_hdr.m_lastsavedby = uh.substr(0,ulen);
+            m_hdr.m_lastsavedby = uh.substr(0, ulen);
             m_hdr.m_lastsavedon = uh.substr(ulen);
           } else
             pws_os::Trace0(_T("FromUTF8(m_wholastsaved) failed\n"));
@@ -798,7 +720,7 @@ int PWSfileV3::ReadHeader()
         break;
 
 #if !defined(USE_XML_LIBRARY) || (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
-      // Don't support importing XML from non-Windows platforms 
+      // Don't support importing XML from non-Windows platforms
       // using Microsoft XML libraries
       // Will be treated as an 'unknown header field' by the 'default' clause below
 #else
@@ -811,18 +733,18 @@ int PWSfileV3::ReadHeader()
           if (!pws_os::FileExists(XSDFilename)) {
             // No filter schema => user won't be able to access stored filters
             // Inform her of the fact (probably an installation problem).
-              stringT message, message2;
-              Format(message, IDSC_MISSINGXSD, _T("pwsafe_filter.xsd"));
-              LoadAString(message2, IDSC_FILTERSKEPT);
-              message += stringT(_T("\n\n")) + message2;
-              if (m_pReporter != NULL)
-                (*m_pReporter)(message);
+            stringT message, message2;
+            Format(message, IDSC_MISSINGXSD, _T("pwsafe_filter.xsd"));
+            LoadAString(message2, IDSC_FILTERSKEPT);
+            message += stringT(_T("\n\n")) + message2;
+            if (m_pReporter != NULL)
+              (*m_pReporter)(message);
 
-              // Treat it as an Unknown field!
-              // Maybe user used a later version of PWS
-              // and we don't want to lose anything
-             UnknownFieldEntry unkhfe(fieldType, utf8Len, utf8);
-             m_UHFL.push_back(unkhfe);
+            // Treat it as an Unknown field!
+            // Maybe user used a later version of PWS
+            // and we don't want to lose anything
+            UnknownFieldEntry unkhfe(fieldType, utf8Len, utf8);
+            m_UHFL.push_back(unkhfe);
             break;
           }
           int rc = m_MapFilters.ImportFilterXMLFile(FPOOL_DATABASE, text.c_str(), _T(""),
@@ -848,7 +770,7 @@ int PWSfileV3::ReadHeader()
         if (utf8 != NULL) utf8[utf8Len] = '\0';
         // All data is character representation of hex - i.e. 0-9a-f
         // No need to convert from char.
-        std::string temp = (char *)utf8;
+        std::string temp = reinterpret_cast<char *>(utf8);
 
         // Get number of entries
         int num(0);
@@ -863,20 +785,89 @@ int PWSfileV3::ReadHeader()
         size_t j = 2;
         for (int n = 0; n < num; n++) {
           unsigned int x(0);
-          uuid_array_t ua;
+          uuid_array_t tmp_ua;
           for (size_t i = 0; i < sizeof(uuid_array_t); i++, j += 2) {
             stringstream ss;
             ss.str(temp.substr(j, 2));
             ss >> hex >> x;
-            ua[i] = static_cast<unsigned char>(x);
+            tmp_ua[i] = static_cast<unsigned char>(x);
           }
-          const CUUID uuid(ua);
+          const CUUID uuid(tmp_ua);
           if (uuid != CUUID::NullUUID())
             m_hdr.m_RUEList.push_back(uuid);
         }
         break;
       }
 
+      case HDR_PSWDPOLICIES:
+      {
+        if (utf8 != NULL) utf8[utf8Len] = '\0';
+        utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
+        if (utf8status) {
+          const size_t recordlength = text.length();
+          StringX sxBlank(_T(" "));  // Needed in case hex value is all zeroes!
+          StringX sxTemp;
+
+          // Get number of polices
+          sxTemp = text.substr(0, 2) + sxBlank;
+          size_t j = 2;  // Skip over # name entries
+          iStringXStream is(sxTemp);
+          int num(0);
+          is >> hex >> num;
+
+          // Get the policies and save them
+          for (int n = 0; n < num; n++) {
+            if (j > recordlength) break;  // Error
+
+            int namelength, symbollength;
+
+            sxTemp = text.substr(j, 2) + sxBlank;
+            iStringXStream tmp_is(sxTemp);
+            j += 2;  // Skip over name length
+
+            tmp_is >> hex >> namelength;
+            if (j + namelength > recordlength) break;  // Error
+
+            StringX sxPolicyName = text.substr(j, namelength);
+            j += namelength;  // Skip over name
+            if (j + 19 > recordlength) break;  // Error
+
+            StringX cs_pwp(text.substr(j, 19));
+            PWPolicy pwp(cs_pwp);
+            j += 19;  // Skip over pwp
+
+            if (j + 2 > recordlength) break;  // Error
+            sxTemp = text.substr(j, 2) + sxBlank;
+            tmp_is.str(sxTemp);
+            j += 2;  // Skip over symbols length
+            tmp_is >> hex >> symbollength;
+
+            StringX sxSymbols;
+            if (symbollength != 0) {
+              if (j + symbollength > recordlength) break;  // Error
+              sxSymbols = text.substr(j, symbollength);
+              j += symbollength;  // Skip over symbols
+            }
+            pwp.symbols = sxSymbols;
+
+            pair< map<StringX, PWPolicy>::iterator, bool > pr;
+            pr = m_MapPSWDPLC.insert(PSWDPolicyMapPair(sxPolicyName, pwp));
+            if (pr.second == false) break; // Error
+          }
+        }
+        break;
+      }
+
+      case HDR_EMPTYGROUP:
+      {
+        if (utf8 != NULL) utf8[utf8Len] = '\0';
+        utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
+        if (utf8status) {
+          m_vEmptyGroups.push_back(text);
+        }
+        break;
+      }
+
       case HDR_END: /* process END so not to treat it as 'unknown' */
         break;
 
@@ -889,7 +880,7 @@ int PWSfileV3::ReadHeader()
 #ifdef _DEBUG
         stringT stimestamp;
         PWSUtil::GetTimeStamp(stimestamp);
-        pws_os::Trace(_T("Header has unknown field: %02x, length %d/0x%04x, value:\n"), 
+        pws_os::Trace(_T("Header has unknown field: %02x, length %d/0x%04x, value:\n"),
                        fieldType, utf8Len, utf8Len);
         pws_os::HexDump(utf8, utf8Len, stimestamp);
 #endif
