@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -20,7 +20,7 @@
 */
 
 /*
-* NOTE: Xerces characters are ALWAYS in UTF-16 (may or may not be wchar_t 
+* NOTE: Xerces characters are ALWAYS in UTF-16 (may or may not be wchar_t
 * depending on platform).
 * Non-unicode builds will need convert any results from parsing the XML
 * document from UTF-16 to ASCII.
@@ -34,7 +34,6 @@
 #include "XFilterSAX2Handlers.h"
 
 #include "../../Util.h"
-#include "../../UUIDGen.h"
 #include "../../core.h"
 #include "../../PWSFilters.h"
 #include "../../VerifyFormat.h"
@@ -56,7 +55,7 @@ using namespace std;
 
 XFilterSAX2Handlers::XFilterSAX2Handlers()
 {
-  m_strElemContent.clear();
+  m_sxElemContent = _T("");
   m_iXMLVersion = -1;
   m_iSchemaVersion = -1;
   m_bErrors = false;
@@ -69,7 +68,7 @@ XFilterSAX2Handlers::~XFilterSAX2Handlers()
 
 void XFilterSAX2Handlers::startDocument()
 {
-  m_strXMLErrors.clear();
+  m_strXMLErrors = _T("");
   m_bEntryBeingProcessed = false;
 }
 
@@ -79,7 +78,7 @@ void XFilterSAX2Handlers::startElement(const XMLCh* const /* uri */,
                                       const Attributes& attrs)
 {
   USES_XMLCH_STR
-  
+
   if (m_bValidation && XMLString::equals(qname, _A2X("filters"))) {
     if (m_pSchema_Version == NULL) {
       LoadAString(m_strXMLErrors, IDSC_MISSING_SCHEMA_VER);
@@ -121,7 +120,7 @@ void XFilterSAX2Handlers::startElement(const XMLCh* const /* uri */,
 
   bool bfilter = XMLString::equals(qname, _A2X("filter"));
   bool bfilter_entry = XMLString::equals(qname, _A2X("filter_entry"));
- 
+
    if (bfilter) {
     cur_filter = new st_filters;
   }
@@ -136,7 +135,7 @@ void XFilterSAX2Handlers::startElement(const XMLCh* const /* uri */,
   if (bfilter || bfilter_entry) {
     // Process the attributes we need.
     if (bfilter) {
-      XMLCh * xmlchValue = (XMLCh *)attrs.getValue(_A2X("filtername")); 
+      const XMLCh * xmlchValue = attrs.getValue(_A2X("filtername"));
       if (xmlchValue != NULL) {
 #ifdef UNICODE
         cur_filter->fname = stringT(_X2ST(xmlchValue));
@@ -149,20 +148,20 @@ void XFilterSAX2Handlers::startElement(const XMLCh* const /* uri */,
     }
 
     if (bfilter_entry) {
-      const XMLCh * xmlchValue = attrs.getValue(_A2X("active")); 
+      const XMLCh * xmlchValue = attrs.getValue(_A2X("active"));
       if (xmlchValue != NULL && XMLString::equals(xmlchValue, _A2X("no")))
         cur_filterentry->bFilterActive = false;
     }
   }
 
-  m_strElemContent = _T("");
+  m_sxElemContent = _T("");
 }
 
 void XFilterSAX2Handlers::characters(const XMLCh* const chars,
                                     const XMLSize_t length)
 {
   USES_XMLCH_STR
-  
+
   if (m_bValidation)
     return;
 
@@ -170,10 +169,10 @@ void XFilterSAX2Handlers::characters(const XMLCh* const chars,
   XMLString::copyNString(xmlchData, chars, length);
   xmlchData[length] = L'\0';
 #ifdef UNICODE
-  m_strElemContent += StringX(_X2SX(xmlchData));
+  m_sxElemContent += StringX(_X2SX(xmlchData));
 #else
   char *szData = XMLString::transcode(xmlchData);
-  m_strElemContent += StringX(szData);
+  m_sxElemContent += StringX(szData);
   XMLString::release(&szData);
 #endif
   delete [] xmlchData;
@@ -183,7 +182,7 @@ void XFilterSAX2Handlers::ignorableWhitespace(const XMLCh* const chars,
                                              const XMLSize_t length)
 {
   USES_XMLCH_STR
-  
+
   if (m_bValidation)
     return;
 
@@ -191,10 +190,10 @@ void XFilterSAX2Handlers::ignorableWhitespace(const XMLCh* const chars,
   XMLString::copyNString(xmlchData, chars, length);
   xmlchData[length] = L'\0';
 #ifdef UNICODE
-  m_strElemContent += StringX(_X2SX(xmlchData));
+  m_sxElemContent += StringX(_X2SX(xmlchData));
 #else
   char *szData = XMLString::transcode(xmlchData);
-  m_strElemContent += StringX(szData);
+  m_sxElemContent += StringX(szData);
   XMLString::release(&szData);
 #endif
   delete [] xmlchData;
@@ -205,7 +204,7 @@ void XFilterSAX2Handlers::endElement(const XMLCh* const /* uri */,
                                     const XMLCh* const qname)
 {
   USES_XMLCH_STR
-  
+
   if (m_bValidation && XMLString::equals(qname, _A2X("filters"))) {
     // Check that the XML file version is present and that
     // a. it is less than or equal to the Filter schema version
@@ -238,7 +237,7 @@ void XFilterSAX2Handlers::endElement(const XMLCh* const /* uri */,
       return ;
     }
     if (m_iXMLVersion > PWS_XML_FILTER_VERSION) {
-      Format(m_strXMLErrors, 
+      Format(m_strXMLErrors,
              IDSC_INVALID_XML_VER2, m_iXMLVersion, PWS_XML_FILTER_VERSION);
 #ifdef UNICODE
       const XMLCh *message = _W2X(m_strXMLErrors.c_str());
@@ -280,8 +279,8 @@ void XFilterSAX2Handlers::endElement(const XMLCh* const /* uri */,
     if (cur_filterentry->mtype  == PWSMatch::MT_DATE &&
         cur_filterentry->rule   != PWSMatch::MR_PRESENT &&
         cur_filterentry->rule   != PWSMatch::MR_NOTPRESENT &&
-        cur_filterentry->fdate1 == (time_t)0 &&
-        cur_filterentry->fdate2 == (time_t)0)
+        cur_filterentry->fdate1 == time_t(0) &&
+        cur_filterentry->fdate2 == time_t(0))
       cur_filterentry->fdatetype = 1; // Relative Date
     if (m_type == DFTYPE_MAIN) {
       cur_filter->num_Mactive++;
@@ -356,6 +355,12 @@ void XFilterSAX2Handlers::endElement(const XMLCh* const /* uri */,
     cur_filterentry->ftype = FT_DCA;
   }
 
+  else if (XMLString::equals(qname, _A2X("shiftdca"))) {
+    m_type = DFTYPE_MAIN;
+    cur_filterentry->mtype = PWSMatch::MT_SHIFTDCA;
+    cur_filterentry->ftype = FT_SHIFTDCA;
+  }
+
   else if (XMLString::equals(qname, _A2X("email"))) {
     m_type = DFTYPE_MAIN;
     cur_filterentry->mtype = PWSMatch::MT_STRING;
@@ -375,6 +380,12 @@ void XFilterSAX2Handlers::endElement(const XMLCh* const /* uri */,
     cur_filterentry->fstring = PWSUtil::DeDupString(cur_filterentry->fstring);
   }
 
+  else if (XMLString::equals(qname, _A2X("policyname"))) {
+    m_type = DFTYPE_MAIN;
+    cur_filterentry->mtype = PWSMatch::MT_STRING;
+    cur_filterentry->ftype = FT_POLICYNAME;
+  }
+
   else if (XMLString::equals(qname, _A2X("create_time"))) {
     m_type = DFTYPE_MAIN;
     cur_filterentry->mtype = PWSMatch::MT_DATE;
@@ -411,6 +422,12 @@ void XFilterSAX2Handlers::endElement(const XMLCh* const /* uri */,
     cur_filterentry->ftype = FT_XTIME_INT;
   }
 
+  else if (XMLString::equals(qname, _A2X("password_length"))) {
+    m_type = DFTYPE_MAIN;
+    cur_filterentry->mtype = PWSMatch::MT_INTEGER;
+    cur_filterentry->ftype = FT_PASSWORDLEN;
+  }
+
   else if (XMLString::equals(qname, _A2X("entrysize"))) {
     m_type = DFTYPE_MAIN;
     cur_filterentry->mtype = PWSMatch::MT_ENTRYSIZE;
@@ -537,84 +554,84 @@ void XFilterSAX2Handlers::endElement(const XMLCh* const /* uri */,
   }
 
   else if (XMLString::equals(qname, _A2X("rule"))) {
-    ToUpper(m_strElemContent);
-    cur_filterentry->rule = PWSMatch::GetRule(m_strElemContent);
+    ToUpper(m_sxElemContent);
+    cur_filterentry->rule = PWSMatch::GetRule(m_sxElemContent.c_str());
   }
 
   else if (XMLString::equals(qname, _A2X("logic"))) {
-    if (m_strElemContent == _T("or"))
+    if (m_sxElemContent == _T("or"))
       cur_filterentry->ltype = LC_OR;
     else
       cur_filterentry->ltype = LC_AND;
   }
 
   else if (XMLString::equals(qname, _A2X("string"))) {
-    cur_filterentry->fstring = m_strElemContent;
+    cur_filterentry->fstring = m_sxElemContent;
   }
 
   else if (XMLString::equals(qname, _A2X("case"))) {
-    cur_filterentry->fcase = _ttoi(m_strElemContent.c_str()) != 0;
+    cur_filterentry->fcase = _ttoi(m_sxElemContent.c_str()) != 0;
   }
 
   else if (XMLString::equals(qname, _A2X("warn"))) {
-    cur_filterentry->fnum1 = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->fnum1 = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (XMLString::equals(qname, _A2X("num1"))) {
-    cur_filterentry->fnum1 = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->fnum1 = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (XMLString::equals(qname, _A2X("num2"))) {
-    cur_filterentry->fnum2 = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->fnum2 = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (XMLString::equals(qname, _A2X("unit"))) {
-    cur_filterentry->funit = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->funit = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (XMLString::equals(qname, _A2X("date1"))) {
     time_t t(0);
-    if (VerifyXMLDateString(m_strElemContent.c_str(), t) &&
-        (t != (time_t)-1))
+    if (VerifyXMLDateString(m_sxElemContent.c_str(), t) &&
+        (t != time_t(-1)))
       cur_filterentry->fdate1 = t;
     else
-    cur_filterentry->fdate1 = (time_t)0;
+    cur_filterentry->fdate1 = time_t(0);
   }
 
   else if (XMLString::equals(qname, _A2X("date2"))) {
     time_t t(0);
-    if (VerifyXMLDateString(m_strElemContent.c_str(), t) &&
-        (t != (time_t)-1))
+    if (VerifyXMLDateString(m_sxElemContent.c_str(), t) &&
+        (t != time_t(-1)))
       cur_filterentry->fdate2 = t;
     else
-      cur_filterentry->fdate1 = (time_t)0;
+      cur_filterentry->fdate1 = time_t(0);
   }
 
   else if (XMLString::equals(qname, _A2X("DCA"))) {
-    cur_filterentry->fdca = (short)_ttoi(m_strElemContent.c_str());
+    cur_filterentry->fdca = static_cast<short>(_ttoi(m_sxElemContent.c_str()));
   }
 
   else if (XMLString::equals(qname, _A2X("type"))) {
-    if (m_strElemContent == _T("normal"))
+    if (m_sxElemContent == _T("normal"))
       cur_filterentry->etype = CItemData::ET_NORMAL;
-    else if (m_strElemContent == _T("alias"))
+    else if (m_sxElemContent == _T("alias"))
       cur_filterentry->etype = CItemData::ET_ALIAS;
-    else if (m_strElemContent == _T("shortcut"))
+    else if (m_sxElemContent == _T("shortcut"))
       cur_filterentry->etype = CItemData::ET_SHORTCUT;
-    else if (m_strElemContent == _T("aliasbase"))
+    else if (m_sxElemContent == _T("aliasbase"))
       cur_filterentry->etype = CItemData::ET_ALIASBASE;
-    else if (m_strElemContent == _T("shortcutbase"))
+    else if (m_sxElemContent == _T("shortcutbase"))
       cur_filterentry->etype = CItemData::ET_SHORTCUTBASE;
     else
       cur_filterentry->etype = CItemData::ET_INVALID;
   }
 
   else if (XMLString::equals(qname, _A2X("status"))) {
-    if (m_strElemContent == _T("clean"))
+    if (m_sxElemContent == _T("clean"))
       cur_filterentry->estatus = CItemData::ES_CLEAN;
-    else if (m_strElemContent == _T("added"))
+    else if (m_sxElemContent == _T("added"))
       cur_filterentry->estatus = CItemData::ES_ADDED;
-    else if (m_strElemContent == _T("modified"))
+    else if (m_sxElemContent == _T("modified"))
       cur_filterentry->estatus = CItemData::ES_MODIFIED;
     else
       cur_filterentry->estatus = CItemData::ES_INVALID;
@@ -632,12 +649,12 @@ void XFilterSAX2Handlers::FormatError(const SAXParseException& e, const int type
   int iLineNumber, iCharacter;
 
 #ifdef UNICODE
-  XMLCh *szErrorMessage = (XMLCh *)e.getMessage();
+  const XMLCh *szErrorMessage = e.getMessage();
 #else
   char *szErrorMessage = XMLString::transcode(e.getMessage());
 #endif
-  iLineNumber = (int)e.getLineNumber();
-  iCharacter = (int)e.getColumnNumber();
+  iLineNumber = static_cast<int>(e.getLineNumber());
+  iCharacter = static_cast<int>(e.getColumnNumber());
 
   stringT cs_format, cs_errortype;
   LoadAString(cs_format, IDSC_XERCESSAXGENERROR);
