@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -26,3 +26,13 @@ bool pws_os::munlock(void *p, size_t size)
   return ::munlock(p, size) == 0;
 }
 
+// Following has OS support only in Windows
+bool pws_os::mcryptProtect(void *, size_t)
+{
+  return true;
+}
+
+bool pws_os::mcryptUnprotect(void *, size_t)
+{
+  return true;
+}
