@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -56,17 +56,12 @@ CAddEdit_Basic::CAddEdit_Basic(CWnd *pParent, st_AE_master_data *pAEMD)
   : CAddEdit_PropertyPage(pParent,
                           CAddEdit_Basic::IDD, CAddEdit_Basic::IDD_SHORT,
                           pAEMD),
-  m_pToolTipCtrl(NULL), m_bInitdone(false), m_thread(NULL)
+  m_bInitdone(false), m_thread(NULL), m_isNotesHidden(false)
 {
   if (CS_SHOW.IsEmpty()) { // one-time initializations
     HIDDEN_NOTES.LoadString(IDS_HIDDENNOTES);
-#if defined(POCKET_PC)
-    CS_SHOW.LoadString(IDS_SHOWPASSWORDTXT1);
-    CS_HIDE.LoadString(IDS_HIDEPASSWORDTXT1);
-#else
-    CS_SHOW.LoadString(IDS_SHOWPASSWORDTXT2);
-    CS_HIDE.LoadString(IDS_HIDEPASSWORDTXT2);
-#endif
+    CS_SHOW.LoadString(IDS_SHOWPASSWORDTXT);
+    CS_HIDE.LoadString(IDS_HIDEPASSWORDTXT);
   }
 
   PWSprefs *prefs = PWSprefs::GetInstance();
@@ -118,7 +113,6 @@ CAddEdit_Basic::CAddEdit_Basic(CWnd *pParent, st_AE_master_data *pAEMD)
 CAddEdit_Basic::~CAddEdit_Basic()
 {
   delete m_pex_notes;
-  delete m_pToolTipCtrl;
 }
 
 void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
@@ -226,40 +220,24 @@ BOOL CAddEdit_Basic::OnInitDialog()
   }
 
   if (M_uicaller() != IDS_ADDENTRY) {
-    m_pToolTipCtrl = new CToolTipCtrl;
-    if (!m_pToolTipCtrl->Create(this, TTS_BALLOON | TTS_NOPREFIX)) {
-      pws_os::Trace(L"Unable To create CAddEdit_Basic Dialog ToolTip\n");
-      delete m_pToolTipCtrl;
-      m_pToolTipCtrl = NULL;
-    } else {
-      EnableToolTips();
+    InitToolTip();
 
-      m_pToolTipCtrl->SetMaxTipWidth(300);
+    AddTool(IDC_STATIC_GROUP,    IDS_CLICKTOCOPY);
+    AddTool(IDC_STATIC_TITLE,    IDS_CLICKTOCOPY);
+    AddTool(IDC_STATIC_USERNAME, IDS_CLICKTOCOPY);
+    AddTool(IDC_STATIC_PASSWORD, IDS_CLICKTOCOPY);
+    AddTool(IDC_STATIC_NOTES,    IDS_CLICKTOCOPY);
+    AddTool(IDC_STATIC_URL,      IDS_CLICKTOCOPY);
+    AddTool(IDC_STATIC_EMAIL,    IDS_CLICKTOCOPYPLUS1);
+    AddTool(IDC_LAUNCH,          IDS_CLICKTOGOPLUS);
+    AddTool(IDC_SENDEMAIL,       IDS_CLICKTOSEND);
 
-      CString cs_ToolTip;
-      cs_ToolTip.LoadString(IDS_CLICKTOCOPY);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_GROUP), cs_ToolTip);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_TITLE), cs_ToolTip);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_USERNAME), cs_ToolTip);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_PASSWORD), cs_ToolTip);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_NOTES), cs_ToolTip);
-      cs_ToolTip.LoadString(IDS_CLICKTOCOPY);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_URL), cs_ToolTip);
-      cs_ToolTip.LoadString(IDS_CLICKTOCOPYPLUS1);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_EMAIL), cs_ToolTip);
-      cs_ToolTip.LoadString(IDS_CLICKTOGOPLUS);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_LAUNCH), cs_ToolTip);
-      cs_ToolTip.LoadString(IDS_CLICKTOSEND);
-      m_pToolTipCtrl->AddTool(GetDlgItem(IDC_SENDEMAIL), cs_ToolTip);
-
-      if (M_uicaller() == IDS_EDITENTRY && M_protected() != 0) {
-        cs_ToolTip.LoadString(IDS_UNPROTECT);
-        m_pToolTipCtrl->AddTool(GetDlgItem(IDC_STATIC_PROTECTED), cs_ToolTip);
-      }
-
-      m_pToolTipCtrl->Activate(TRUE);
+    if (M_uicaller() == IDS_EDITENTRY && M_protected() != 0) {
+      AddTool(IDC_STATIC_PROTECTED, IDS_UNPROTECT);
     }
 
+    ActivateToolTip();
+
     m_stc_group.SetHighlight(true, CAddEdit_PropertyPage::crefWhite);
     m_stc_title.SetHighlight(true, CAddEdit_PropertyPage::crefWhite);
     m_stc_username.SetHighlight(true, CAddEdit_PropertyPage::crefWhite);
@@ -369,6 +347,8 @@ BOOL CAddEdit_Basic::OnInitDialog()
     ShowNotes();
   } else {
     HideNotes();
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
   }
 
   CHARFORMAT cf = {0};
@@ -377,15 +357,6 @@ BOOL CAddEdit_Basic::OnInitDialog()
   m_pex_notes->SetSel(0, -1);
   m_pex_notes->SendMessage(EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
   m_iPointSize = cf.yHeight / 20;
-  m_pex_notes->Clear();
-
-  if (m_isNotesHidden) {
-    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
-    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
-  } else {
-    // If at the limit - don't allow to be called again in that direction
-    SetZoomMenu();
-  }
 
   // Set initial Word Wrap
   m_pex_notes->SetTargetDevice(NULL, m_bWordWrap ? 0 : 1);
@@ -398,9 +369,7 @@ BOOL CAddEdit_Basic::OnInitDialog()
 
 void CAddEdit_Basic::OnHelp()
 {
-  CString cs_HelpTopic;
-  cs_HelpTopic = app.GetHelpFileName() + L"::/html/entering_pwd.html";
-  HtmlHelp(DWORD_PTR((LPCWSTR)cs_HelpTopic), HH_DISPLAY_TOPIC);
+  ShowHelp(L"::/html/entering_pwd.html");
 }
 
 HBRUSH CAddEdit_Basic::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
@@ -487,8 +456,9 @@ LRESULT CAddEdit_Basic::OnQuerySiblings(WPARAM wParam, LPARAM )
               M_realnotes()    != M_originalrealnotesTRC() ||
               M_URL()          != M_pci()->GetURL()        ||
               M_email()        != M_pci()->GetEmail()      ||
-              M_symbols()      != M_pci()->GetSymbols()    ||
-              M_realpassword() != M_oldRealPassword())
+              (M_ipolicy() != NAMED_POLICY &&
+               M_symbols()      != M_pci()->GetSymbols())  ||
+              M_realpassword() != M_oldRealPassword()        )
             return 1L;
           break;
         case IDS_ADDENTRY:
@@ -521,9 +491,7 @@ BOOL CAddEdit_Basic::PreTranslateMessage(MSG* pMsg)
     return TRUE;
   }
 
-  // Do tooltips
-  if (m_pToolTipCtrl != NULL)
-    m_pToolTipCtrl->RelayEvent(pMsg);
+  RelayToolTipEvent(pMsg);
 
   // Ctrl + 'key' in Notes
   if (pMsg->message == WM_KEYDOWN &&
@@ -753,8 +721,9 @@ void CAddEdit_Basic::HidePassword()
 
 void CAddEdit_Basic::SetZoomMenu()
 {
-    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, m_iPointSize < 72);
-    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, m_iPointSize > 6);
+  // If at the limit - don't allow to be called again in that direction
+  m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, m_iPointSize < 72);
+  m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, m_iPointSize > 6);
 }
 
 LRESULT CAddEdit_Basic::OnZoomNotes(WPARAM, LPARAM lParam)
@@ -790,27 +759,26 @@ LRESULT CAddEdit_Basic::OnZoomNotes(WPARAM, LPARAM lParam)
 
 void CAddEdit_Basic::ShowNotes()
 {
-  m_isNotesHidden = false;
-  m_notes = M_realnotes();
-
-  ((CEdit *)GetDlgItem(IDC_NOTES))->Invalidate();
-
-  // If at the limit - don't allow to be called again in that direction
-  SetZoomMenu();
+  if (m_isNotesHidden) { // idempotent
+    m_isNotesHidden = false;
+    m_notes = M_realnotes();
+    GetDlgItem(IDC_NOTES)->Invalidate();
+    SetZoomMenu();
+  }
 }
 
 void CAddEdit_Basic::HideNotes()
 {
-  m_isNotesHidden = true;
-  M_realnotes() = m_notes;
-
-  m_notes = HIDDEN_NOTES;
-
-  ((CEdit *)GetDlgItem(IDC_NOTES))->Invalidate();
-
-  // Disable zoom of hidden text
-  m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
-  m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
+  if (!m_isNotesHidden) { // idempotent
+    m_isNotesHidden = true;
+    M_realnotes() = m_notes;
+    
+    m_notes = HIDDEN_NOTES;
+    GetDlgItem(IDC_NOTES)->Invalidate();
+    // Disable zoom of hidden text
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
+  }
 }
 
 void CAddEdit_Basic::OnGeneratePassword()
@@ -997,16 +965,14 @@ LRESULT CAddEdit_Basic::OnWordWrap(WPARAM, LPARAM)
 void CAddEdit_Basic::OnENSetFocusNotes()
 {
   UpdateData(TRUE);
-  if (m_isNotesHidden) {
-    ShowNotes();
-  }
+  ShowNotes();
   UpdateData(FALSE);
 }
 
 void CAddEdit_Basic::OnENKillFocusNotes()
 {
   UpdateData(TRUE);
-  if (!m_isNotesHidden) {
+  if (!PWSprefs::GetInstance()->GetPref(PWSprefs::ShowNotesDefault)) {
     HideNotes();
   } else
     M_realnotes() = m_notes;
@@ -1024,6 +990,8 @@ void CAddEdit_Basic::OnLaunch()
                                                        M_username(),
                                                        M_realpassword(),
                                                        M_realnotes(),
+                                                       M_URL(),
+                                                       M_email(),
                                                        vactionverboffsets);
 
   const bool bDoAutoType = (GetKeyState(VK_CONTROL) & 0x8000) != 0;
@@ -1224,7 +1192,7 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM, LPARAM)
   M_realnotes() = m_notes = note.c_str();
 
   UpdateData(FALSE);
-  ((CEdit*)GetDlgItem(IDC_NOTES))->Invalidate();
+  GetDlgItem(IDC_NOTES)->Invalidate();
 
   // Delete temporary file
   _wremove(m_szTempName);
