@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -15,15 +15,19 @@
 #include "core/ItemData.h"
 #include "core/PWSprefs.h"
 
+using pws_os::CUUID;
+
 IMPLEMENT_DYNAMIC(CAddEdit_PropertySheet, CPWPropertySheet)
 
 CAddEdit_PropertySheet::CAddEdit_PropertySheet(UINT nID, CWnd* pParent,
-                                               PWScore *pcore, 
+                                               PWScore *pcore,
                                                CItemData *pci_original, CItemData *pci,
+                                               const bool bLongPPs,
                                                const StringX currentDB)
-  : CPWPropertySheet(nID, pParent), m_bIsModified(false), m_bChanged(false),
-  m_bNotesChanged(false)
+  : CPWPropertySheet(nID, pParent, bLongPPs), m_bIsModified(false), m_bChanged(false),
+  m_bNotesChanged(false), m_bSymbolsChanged(false)
 {
+  m_AEMD.bLongPPs = bLongPPs;
   m_AEMD.uicaller = nID;
 
   ASSERT(pParent != NULL);
@@ -39,27 +43,8 @@ CAddEdit_PropertySheet::CAddEdit_PropertySheet(UINT nID, CWnd* pParent,
 
   PWSprefs *prefs = PWSprefs::GetInstance();
 
-  m_AEMD.default_pwp.Empty();
-  if (prefs->GetPref(PWSprefs::PWUseLowercase))
-    m_AEMD.default_pwp.flags |= PWSprefs::PWPolicyUseLowercase;
-  if (prefs->GetPref(PWSprefs::PWUseUppercase))
-    m_AEMD.default_pwp.flags |= PWSprefs::PWPolicyUseUppercase;
-  if (prefs->GetPref(PWSprefs::PWUseDigits))
-    m_AEMD.default_pwp.flags |= PWSprefs::PWPolicyUseDigits;
-  if (prefs->GetPref(PWSprefs::PWUseSymbols))
-    m_AEMD.default_pwp.flags |= PWSprefs::PWPolicyUseSymbols;
-  if (prefs->GetPref(PWSprefs::PWUseHexDigits))
-    m_AEMD.default_pwp.flags |= PWSprefs::PWPolicyUseHexDigits;
-  if (prefs->GetPref(PWSprefs::PWUseEasyVision))
-    m_AEMD.default_pwp.flags |= PWSprefs::PWPolicyUseEasyVision;
-  if (prefs->GetPref(PWSprefs::PWMakePronounceable))
-    m_AEMD.default_pwp.flags |= PWSprefs::PWPolicyMakePronounceable;
-
-  m_AEMD.default_pwp.length = prefs->GetPref(PWSprefs::PWDefaultLength);
-  m_AEMD.default_pwp.digitminlength = prefs->GetPref(PWSprefs::PWDigitMinLength);
-  m_AEMD.default_pwp.lowerminlength = prefs->GetPref(PWSprefs::PWLowercaseMinLength);
-  m_AEMD.default_pwp.symbolminlength = prefs->GetPref(PWSprefs::PWSymbolMinLength);
-  m_AEMD.default_pwp.upperminlength = prefs->GetPref(PWSprefs::PWUppercaseMinLength);
+  m_AEMD.default_pwp = prefs->GetDefaultPolicy();
+  m_AEMD.default_symbols = prefs->GetPref(PWSprefs::DefaultSymbols);
 
   // Set up data used by all Property Pages, as appropriate
   if (m_AEMD.uicaller == IDS_ADDENTRY) {
@@ -79,11 +64,11 @@ CAddEdit_PropertySheet::CAddEdit_PropertySheet(UINT nID, CWnd* pParent,
     // Additional initialisation
     m_AEMD.autotype = L"";
     m_AEMD.runcommand = L"";
-    m_AEMD.oldDCA = m_AEMD.DCA = -1;
+    m_AEMD.oldDCA = m_AEMD.DCA = m_AEMD.oldShiftDCA = m_AEMD.ShiftDCA = -1;
 
     // Date & Time initialisation
     m_AEMD.locCTime.LoadString(IDS_NA);
-    m_AEMD.locXTime = m_AEMD.locATime = m_AEMD.locRMTime = m_AEMD.locPMTime = 
+    m_AEMD.locXTime = m_AEMD.locATime = m_AEMD.locRMTime = m_AEMD.locPMTime =
           m_AEMD.oldlocXTime = m_AEMD.locCTime;
     m_AEMD.tttXTime = m_AEMD.tttCPMTime = (time_t)0;
     m_AEMD.oldXTimeInt = m_AEMD.XTimeInt = 0;
@@ -100,6 +85,7 @@ CAddEdit_PropertySheet::CAddEdit_PropertySheet(UINT nID, CWnd* pParent,
     m_AEMD.ipolicy = m_AEMD.oldipolicy = DEFAULT_POLICY;
     m_AEMD.iownsymbols = m_AEMD.ioldownsymbols = DEFAULT_SYMBOLS;
     m_AEMD.symbols = L"";
+    m_AEMD.policyname = m_AEMD.oldpolicyname = L"";
 
     // Protected
     m_AEMD.ucprotected = 0;
@@ -127,9 +113,27 @@ CAddEdit_PropertySheet::~CAddEdit_PropertySheet()
   delete m_pp_pwpolicy;
 }
 
+BEGIN_MESSAGE_MAP(CAddEdit_PropertySheet, CPWPropertySheet)
+  //{{AFX_MSG_MAP(CAddEdit_PropertySheet)
+  ON_WM_SYSCOMMAND()
+  //}}AFX_MSG_MAP
+END_MESSAGE_MAP()
+
+void CAddEdit_PropertySheet::OnSysCommand(UINT nID, LPARAM lParam)
+{
+  const UINT nSysID = nID & 0xFFF0;
+  if (nSysID == SC_CLOSE &&
+      (m_AEMD.uicaller == IDS_VIEWENTRY ||
+      (m_AEMD.uicaller == IDS_EDITENTRY &&  m_AEMD.ucprotected != 0))) {
+    EndDialog(IDCANCEL);
+    return;
+  }
+  CPWPropertySheet::OnSysCommand(nID, lParam);
+}
+
 BOOL CAddEdit_PropertySheet::OnInitDialog()
 {
-  CPropertySheet::OnInitDialog();
+  CPWPropertySheet::OnInitDialog();
 
   // Change the Window title for Edit/View
   switch (m_AEMD.uicaller) {
@@ -138,7 +142,7 @@ BOOL CAddEdit_PropertySheet::OnInitDialog()
     case IDS_VIEWENTRY:
     case IDS_EDITENTRY:
       {
-      CString cs_title;
+        CString cs_title;
       StringX sx_group(L""), sx_title, sx_user(L"");
       if (!m_AEMD.pci->IsGroupEmpty())
         sx_group = m_AEMD.pci->GetGroup();
@@ -147,7 +151,9 @@ BOOL CAddEdit_PropertySheet::OnInitDialog()
         sx_user = m_AEMD.pci->GetUser();
 
       // Set up and pass Propertysheet caption showing entry being edited/viewed
-      cs_title.Format(m_AEMD.uicaller, sx_group.c_str(), sx_title.c_str(), sx_user.c_str());
+      // If entry is protected, set to 'View' even if DB is in R/W mode
+      cs_title.Format(m_AEMD.ucprotected != 0 ? IDS_PROTECTEDENTRY : m_AEMD.uicaller,
+                      sx_group.c_str(), sx_title.c_str(), sx_user.c_str());
       SetWindowText(cs_title);
       break;
       }
@@ -164,19 +170,58 @@ BOOL CAddEdit_PropertySheet::OnInitDialog()
       GetDlgItem(IDCANCEL)->ShowWindow(SW_HIDE);
       break;
     case IDS_EDITENTRY:
-      GetDlgItem(IDOK)->EnableWindow((m_bChanged || m_AEMD.ucprotected != 0) ? TRUE : FALSE);
-      GetDlgItem(ID_APPLY_NOW)->EnableWindow(m_bChanged ? TRUE : FALSE);
+      GetDlgItem(ID_APPLY_NOW)->EnableWindow(m_bChanged || m_bSymbolsChanged ? TRUE : FALSE);
       break;
   }
   return TRUE;
 }
 
+void CAddEdit_PropertySheet::SetSymbolsChanged(const bool bSymbolsChanged)
+{
+  m_bSymbolsChanged = bSymbolsChanged;
+  bool bChanged = m_bChanged || m_bSymbolsChanged;
+
+  GetDlgItem(IDOK)->EnableWindow(bChanged ? TRUE : FALSE);
+  if (m_AEMD.uicaller == IDS_EDITENTRY)
+    GetDlgItem(ID_APPLY_NOW)->EnableWindow(bChanged ? TRUE : FALSE);
+}
+
 void CAddEdit_PropertySheet::SetChanged(const bool bChanged)
 {
   if (m_bChanged != bChanged) {
-    GetDlgItem(IDOK)->EnableWindow(bChanged ? TRUE : FALSE);
     if (m_AEMD.uicaller == IDS_EDITENTRY)
       GetDlgItem(ID_APPLY_NOW)->EnableWindow(bChanged ? TRUE : FALSE);
+    switch (m_AEMD.uicaller) {
+    case IDS_ADDENTRY:
+      break;
+    case IDS_VIEWENTRY:
+    case IDS_EDITENTRY:
+      {
+        CString cs_title;
+        StringX sx_group(L""), sx_title, sx_user(L"");
+        if (!m_AEMD.pci->IsGroupEmpty())
+          sx_group = m_AEMD.pci->GetGroup();
+        sx_title = m_AEMD.pci->GetTitle();
+        if (!m_AEMD.pci->IsUserEmpty())
+          sx_user = m_AEMD.pci->GetUser();
+
+        // Set up and pass Propertysheet caption showing entry being edited/viewed
+        // If entry is protected, set to 'View' even if DB is in R/W mode
+        cs_title.Format(m_AEMD.ucprotected != 0 ? IDS_PROTECTEDENTRY : m_AEMD.uicaller,
+                        sx_group.c_str(), sx_title.c_str(), sx_user.c_str());
+
+        const CString asterisk = L"*";
+        if (bChanged)
+          cs_title += asterisk;
+        else
+          cs_title.TrimRight(asterisk[0]);
+
+        SetWindowText(cs_title);
+        break;
+      }
+    default:
+      ASSERT(0);
+    }
     m_bChanged = bChanged;
   }
 }
@@ -186,7 +231,11 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
   // There is no OnOK for classes derived from CPropertySheet,
   // so we make our own!
   const int iCID = LOWORD(wParam);
-  if (iCID == IDOK || iCID == ID_APPLY_NOW) {
+  if (HIWORD(wParam) == BN_CLICKED && (iCID == IDOK || iCID == ID_APPLY_NOW)) {
+    // Don't care what user has done if entry is protected or DB R-O.
+    if (m_AEMD.ucprotected != 0 || m_AEMD.uicaller == IDS_VIEWENTRY)
+      CPWPropertySheet::EndDialog(IDOK);
+
     // First send a message to all loaded pages using base class function.
     // We want them all to update their variables in the Master Data area.
     // And call OnApply() rather than the default OnOK processing
@@ -194,12 +243,13 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
     // loaded - i.e. the user has selected to view them, since obviously
     // the user would not have changed their values if not displayed. Duh!
     if (SendMessage(PSM_QUERYSIBLINGS,
-                (WPARAM)CPWPropertyPage::PP_UPDATE_VARIABLES, 0L) != 0)
+                (WPARAM)CPWPropertyPage::PP_UPDATE_VARIABLES, 0L) != 0) {
       return TRUE;
+    }
 
     time_t t;
     bool bIsPSWDModified;
-    short iDCA;
+    short iDCA, iShiftDCA;
 
     switch (m_AEMD.uicaller) {
       case IDS_EDITENTRY:
@@ -208,6 +258,7 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
           break;
 
         m_AEMD.pci->GetDCA(iDCA);
+        m_AEMD.pci->GetShiftDCA(iShiftDCA);
         // Check if modified
         m_bIsModified = (m_AEMD.group       != m_AEMD.pci->GetGroup()      ||
                          m_AEMD.title       != m_AEMD.pci->GetTitle()      ||
@@ -217,6 +268,7 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
                          m_AEMD.autotype    != m_AEMD.pci->GetAutoType()   ||
                          m_AEMD.runcommand  != m_AEMD.pci->GetRunCommand() ||
                          m_AEMD.DCA         != iDCA                        ||
+                         m_AEMD.ShiftDCA    != iShiftDCA                   ||
                          m_AEMD.email       != m_AEMD.pci->GetEmail()      ||
                          m_AEMD.symbols     != m_AEMD.oldsymbols           ||
                          m_AEMD.PWHistory   != m_AEMD.pci->GetPWHistory()  ||
@@ -224,7 +276,9 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
                          m_AEMD.XTimeInt    != m_AEMD.oldXTimeInt          ||
                          m_AEMD.ipolicy     != m_AEMD.oldipolicy           ||
                         (m_AEMD.ipolicy     == SPECIFIC_POLICY &&
-                         m_AEMD.pwp         != m_AEMD.oldpwp));
+                         m_AEMD.pwp         != m_AEMD.oldpwp)              ||
+                        (m_AEMD.ipolicy     == NAMED_POLICY &&
+                         m_AEMD.policyname  != m_AEMD.oldpolicyname));
 
         bIsPSWDModified = (m_AEMD.realpassword != m_AEMD.oldRealPassword);
 
@@ -240,25 +294,41 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
           m_AEMD.pci->SetURL(m_AEMD.URL);
           m_AEMD.pci->SetAutoType(m_AEMD.autotype);
           m_AEMD.pci->SetPWHistory(m_AEMD.PWHistory);
-
-          if (m_AEMD.ipolicy == DEFAULT_POLICY)
-            m_AEMD.pci->SetPWPolicy(L"");
-          else
-            m_AEMD.pci->SetPWPolicy(m_AEMD.pwp);
+          m_AEMD.PWHistory = m_AEMD.PWHistory;
+          m_AEMD.oldNumPWHistory = m_AEMD.NumPWHistory;
+          m_AEMD.oldMaxPWHistory = m_AEMD.MaxPWHistory;
+          m_AEMD.oldSavePWHistory = m_AEMD.SavePWHistory;		 
+          switch (m_AEMD.ipolicy) {
+            case DEFAULT_POLICY:
+              m_AEMD.pci->SetPWPolicy(L"");
+              m_AEMD.policyname = L"";
+              m_AEMD.pci->SetPolicyName(L"");
+              break;
+            case NAMED_POLICY:
+              m_AEMD.pci->SetPWPolicy(L"");
+              m_AEMD.pci->SetPolicyName(m_AEMD.policyname);
+              break;
+            case SPECIFIC_POLICY:
+              m_AEMD.pci->SetPWPolicy(m_AEMD.pwp);
+              m_AEMD.policyname = L"";
+              m_AEMD.pci->SetPolicyName(L"");
+              break;
+           }
 
           m_AEMD.oldipolicy = m_AEMD.ipolicy;
           m_AEMD.oldpwp = m_AEMD.pwp;
           m_AEMD.oldsymbols = m_AEMD.symbols;
+          m_AEMD.oldpolicyname = m_AEMD.policyname;
 
           m_AEMD.pci->SetRunCommand(m_AEMD.runcommand);
           m_AEMD.pci->SetDCA(m_AEMD.DCA);
+          m_AEMD.pci->SetShiftDCA(m_AEMD.ShiftDCA);
           m_AEMD.pci->SetEmail(m_AEMD.email);
           m_AEMD.pci->SetSymbols(m_AEMD.symbols);
           m_AEMD.pci->SetProtected(m_AEMD.ucprotected != 0);
         }
 
-        if (m_AEMD.XTimeInt > 0 && m_AEMD.XTimeInt <= 3650)
-          m_AEMD.pci->SetXTimeInt(m_AEMD.XTimeInt);
+        m_AEMD.pci->SetXTimeInt(m_AEMD.XTimeInt);
 
         if (bIsPSWDModified || m_AEMD.locXTime != m_AEMD.oldlocXTime) {
           CItemData *pciA(m_AEMD.pci);
@@ -302,6 +372,7 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
         m_AEMD.pci->SetAutoType(m_AEMD.autotype);
         m_AEMD.pci->SetRunCommand(m_AEMD.runcommand);
         m_AEMD.pci->SetDCA(m_AEMD.DCA);
+        m_AEMD.pci->SetShiftDCA(m_AEMD.ShiftDCA);
         m_AEMD.pci->SetEmail(m_AEMD.email);
         m_AEMD.pci->SetSymbols(m_AEMD.symbols);
         m_AEMD.pci->SetProtected(m_AEMD.ucprotected != 0);
@@ -334,10 +405,23 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
           m_AEMD.pci->SetPWPolicy(L"");
         } else {
           m_AEMD.pci->SetXTime(m_AEMD.tttXTime);
-          if (m_AEMD.ipolicy == DEFAULT_POLICY)
-            m_AEMD.pci->SetPWPolicy(L"");
-          else
-            m_AEMD.pci->SetPWPolicy(m_AEMD.pwp);
+
+          switch (m_AEMD.ipolicy) {
+            case DEFAULT_POLICY:
+              m_AEMD.pci->SetPWPolicy(L"");
+              m_AEMD.policyname = L"";
+              m_AEMD.pci->SetPolicyName(L"");
+              break;
+            case NAMED_POLICY:
+              m_AEMD.pci->SetPWPolicy(L"");
+              m_AEMD.pci->SetPolicyName(m_AEMD.policyname);
+              break;
+            case SPECIFIC_POLICY:
+              m_AEMD.pci->SetPWPolicy(m_AEMD.pwp);
+              m_AEMD.policyname = L"";
+              m_AEMD.pci->SetPolicyName(L"");
+              break;
+          }
         }
 
         if (m_bIsModified)
@@ -375,7 +459,7 @@ BOOL CAddEdit_PropertySheet::OnCommand(WPARAM wParam, LPARAM lParam)
   return CPWPropertySheet::OnCommand(wParam, lParam);
 }
 
-BOOL CAddEdit_PropertySheet::PreTranslateMessage(MSG* pMsg) 
+BOOL CAddEdit_PropertySheet::PreTranslateMessage(MSG* pMsg)
 {
   // In View mode, there is no 'Cancel' button and 'OK' is renamed 'Close'
   // Make Escape key still work as designed
@@ -389,7 +473,7 @@ BOOL CAddEdit_PropertySheet::PreTranslateMessage(MSG* pMsg)
       BOOL brc = pp->OnQueryCancel();
       if (brc == TRUE)
         CPWPropertySheet::EndDialog(IDCANCEL);
-      
+     
       return TRUE;
     }
   }
@@ -407,7 +491,7 @@ void CAddEdit_PropertySheet::SetupInitialValues()
 {
   // Basic Data
   m_AEMD.entrysize = m_AEMD.pci->GetSize();
-  m_AEMD.pci->GetUUID(m_AEMD.entry_uuid);
+  m_AEMD.entry_uuid = m_AEMD.pci->GetUUID();
   m_AEMD.group = m_AEMD.pci->GetGroup();
   m_AEMD.title = m_AEMD.pci->GetTitle();
   m_AEMD.username = m_AEMD.pci->GetUser();
@@ -437,6 +521,10 @@ void CAddEdit_PropertySheet::SetupInitialValues()
   const CItemData *pciA(m_AEMD.pci);
   if (m_AEMD.pci->IsAlias()) {
     pciA = m_AEMD.pcore->GetBaseEntry(m_AEMD.pci);
+    if (pciA == NULL) {
+      CString err(MAKEINTRESOURCE(IDS_ABASE_MISSING));
+      throw err;
+    }
   }
 
   // Additional data
@@ -444,6 +532,8 @@ void CAddEdit_PropertySheet::SetupInitialValues()
   m_AEMD.runcommand = m_AEMD.pci->GetRunCommand();
   m_AEMD.pci->GetDCA(m_AEMD.DCA);
   m_AEMD.oldDCA = m_AEMD.DCA;
+  m_AEMD.pci->GetShiftDCA(m_AEMD.ShiftDCA);
+  m_AEMD.oldShiftDCA = m_AEMD.ShiftDCA;
 
   // Date Time fields
   m_AEMD.locCTime = m_AEMD.pci->GetCTimeL();
@@ -486,21 +576,29 @@ void CAddEdit_PropertySheet::SetupInitialValues()
   // Note different pci depending on if Alias
   size_t num_err;
   m_AEMD.PWHistory = pciA->GetPWHistory();
-
   BOOL HasHistory = CreatePWHistoryList(m_AEMD.PWHistory,
                                         m_AEMD.MaxPWHistory,
                                         num_err,
                                         m_AEMD.pwhistlist,
-                                        TMC_EXPORT_IMPORT) ? TRUE : FALSE;
+                                        PWSUtil::TMC_EXPORT_IMPORT) ? TRUE : FALSE;
   m_AEMD.oldNumPWHistory = m_AEMD.NumPWHistory = m_AEMD.pwhistlist.size();
   m_AEMD.oldSavePWHistory = m_AEMD.SavePWHistory = HasHistory;
-
+  if (m_AEMD.MaxPWHistory == 0)
+    m_AEMD.MaxPWHistory = PWSprefs::GetInstance()->
+                           GetPref(PWSprefs::NumPWHistoryDefault);
   // PWPolicy fields
   // Note different pci depending on if Alias
   pciA->GetPWPolicy(m_AEMD.pwp);
+  m_AEMD.policyname = m_AEMD.oldpolicyname = pciA->GetPolicyName();
+
+  if (!m_AEMD.policyname.IsEmpty())
+    m_AEMD.ipolicy = NAMED_POLICY;
+  else
+  if (pciA->GetPWPolicy().empty())
+    m_AEMD.ipolicy = DEFAULT_POLICY;
+  else
+    m_AEMD.ipolicy = SPECIFIC_POLICY;
 
-  m_AEMD.ipolicy = (pciA->GetPWPolicy().empty()) ?
-                             DEFAULT_POLICY : SPECIFIC_POLICY;
   m_AEMD.oldipolicy = m_AEMD.ipolicy;
 
   if (m_AEMD.ipolicy == DEFAULT_POLICY) {
@@ -510,10 +608,10 @@ void CAddEdit_PropertySheet::SetupInitialValues()
   m_AEMD.oldpwp = m_AEMD.pwp;
 
   // Set up dependents
-  uuid_array_t original_uuid = {'\0'}, original_base_uuid = {'\0'};
+  pws_os::CUUID original_base_uuid(pws_os::CUUID::NullUUID());
   CItemData::EntryType entrytype = m_AEMD.pci_original->GetEntryType();
 
-  m_AEMD.pci_original->GetUUID(original_uuid);  // Edit doesn't change this!
+  pws_os::CUUID original_uuid = m_AEMD.pci_original->GetUUID();  // Edit doesn't change this!
   if (m_AEMD.pci_original->IsBase()) {
     UUIDVector dependentslist;
     StringX csDependents(L"");
@@ -536,8 +634,8 @@ void CAddEdit_PropertySheet::SetupInitialValues()
     const CItemData *pbci = m_AEMD.pcore->GetBaseEntry(m_AEMD.pci_original);
     ASSERT(pbci != NULL);
     if (pbci != NULL) {
-      pbci->GetUUID(original_base_uuid);
-      memcpy(m_AEMD.base_uuid, original_base_uuid, sizeof(uuid_array_t));
+      original_base_uuid = pbci->GetUUID();
+      m_AEMD.base_uuid = original_base_uuid;
       CSecString cs_base = L"[" +
                            pbci->GetGroup() + L":" +
                            pbci->GetTitle() + L":" +
