@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -25,6 +25,7 @@ class CAddEdit_PropertySheet : public CPWPropertySheet
 public:
   CAddEdit_PropertySheet(UINT nID, CWnd* pDbx, PWScore *pcore, 
                          CItemData *pci_original, CItemData *pci,
+                         const bool bLongPPs,
                          const StringX currentDB = L"");
   ~CAddEdit_PropertySheet();
 
@@ -69,6 +70,10 @@ public:
 protected:
   st_AE_master_data m_AEMD;
 
+  afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
+
+  DECLARE_MESSAGE_MAP()
+
 private:
   void SetupInitialValues();
 
