@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -12,14 +12,9 @@
 
 #include "PWSclipboard.h"
 
-#if defined(POCKET_PC)
-#include "pocketpc/resource.h"
-#include "pocketpc/MyListCtrl.h"
-#else
 #include "resource.h"
 #include "resource2.h"  // Version, Menu, Toolbar & Accelerator resources
 #include "resource3.h"  // String resources
-#endif
 
 #include "PWTreeCtrl.h"
 #include "PWListCtrl.h"
@@ -70,7 +65,7 @@
 
 #endif  /* WINVER < 0x0501 */
 
-#if defined(POCKET_PC) || (_MFC_VER <= 1200)
+#if (_MFC_VER <= 1200)
 DECLARE_HANDLE(HDROP);
 #endif
 
@@ -203,10 +198,6 @@ class CAddEdit_PropertySheet;
 //-----------------------------------------------------------------------------
 class DboxMain : public CDialog, public UIInterFace
 {
-#if defined(POCKET_PC)
-  friend class CMyListCtrl;
-#endif
-
 public:
   DECLARE_DYNAMIC(DboxMain)
 
@@ -275,16 +266,17 @@ public:
   void ChangeOkUpdate();
 
   // when Group, Title or User edited in tree
-  void UpdateListItem(const int lindex, const int type, const StringX &sxnewText);
-  void UpdateTreeItem(const HTREEITEM hItem, const StringX &sxnewText);
+  void UpdateListItemField(const int lindex, const int type, const StringX &sxnewText);
+  void UpdateTreeItem(const HTREEITEM hItem, const CItemData &ci);
   void UpdateListItemGroup(const int lindex, const StringX &sxnewGroup)
-  {UpdateListItem(lindex, CItemData::GROUP, sxnewGroup);}
+  {UpdateListItemField(lindex, CItemData::GROUP, sxnewGroup);}
   void UpdateListItemTitle(const int lindex, const StringX &sxnewTitle)
-  {UpdateListItem(lindex, CItemData::TITLE, sxnewTitle);}
+  {UpdateListItemField(lindex, CItemData::TITLE, sxnewTitle);}
   void UpdateListItemUser(const int lindex, const StringX &sxnewUser)
-  {UpdateListItem(lindex, CItemData::USER, sxnewUser);}
+  {UpdateListItemField(lindex, CItemData::USER, sxnewUser);}
   void UpdateListItemPassword(const int lindex, const StringX &sxnewPassword)
-  {UpdateListItem(lindex, CItemData::PASSWORD, sxnewPassword);}
+  {UpdateListItemField(lindex, CItemData::PASSWORD, sxnewPassword);}
+  
   void SetHeaderInfo();
   CString GetHeaderText(int iType) const;
   int GetHeaderWidth(int iType) const;
@@ -431,11 +423,7 @@ public:
 
   //{{AFX_DATA(DboxMain)
   enum { IDD = IDD_PASSWORDSAFE_DIALOG };
-#if defined(POCKET_PC)
-  CMyListCtrl m_ctlItemList;
-#else
   CPWListCtrl m_ctlItemList;
-#endif
   CPWToolBar m_MainToolBar;   // main toolbar
   CPWTreeCtrl m_ctlItemTree;
   CImageList *m_pImageList;
@@ -481,7 +469,10 @@ public:
   
   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(DboxMain)
-protected:
+
+  void ChangeMode(bool promptUser); // r-o <-> r/w
+  
+ protected:
   virtual void DoDataExchange(CDataExchange* pDX);  // DDX/DDV support
   //}}AFX_VIRTUAL
 
@@ -498,21 +489,14 @@ protected:
   bool m_bNoValidation;   // invoked with '--novalidate'?
   bool m_bCompareEntries; // invoked with '--cetreeview'?
   
-#if !defined(POCKET_PC)
   CString m_titlebar; // what's displayed in the title bar
-#endif
 
-#if defined(POCKET_PC)
-  CCeCommandBar *m_wndCommandBar;
-  CMenu *m_pwndMenu;
-#else
   CPWFindToolBar m_FindToolBar;  // Find toolbar
   CPWStatusBar m_statusBar;
   BOOL m_toolbarsSetup;
   UINT m_toolbarMode;
   UINT statustext[CPWStatusBar::SB_TOTAL];
   CString m_lastclipboardaction;
-#endif
 
   bool m_bInitDone;
   bool m_bDBNeedsReading;
@@ -630,7 +614,6 @@ protected:
   void SaveGUIStatus();
   void RestoreGUIStatus();
 
-#if !defined(POCKET_PC)
   afx_msg void OnTrayLockUnLock();
   afx_msg void OnTrayClearRecentEntries();
   afx_msg void OnUpdateTrayClearRecentEntries(CCmdUI *pCmdUI);
@@ -656,7 +639,6 @@ protected:
   afx_msg void OnUpdateTraySendEmail(CCmdUI *pCmdUI);
   afx_msg void OnTraySelect(UINT nID);
   afx_msg void OnUpdateTraySelect(CCmdUI *pCmdUI);
-#endif
 
   // Generated message map functions
   //{{AFX_MSG(DboxMain)
@@ -760,11 +742,7 @@ protected:
   afx_msg void OnRunCommand();
   afx_msg void OnColumnPicker();
   afx_msg void OnResetColumns();
-#if defined(POCKET_PC)
-  afx_msg void OnShowPassword();
-#else
   afx_msg void OnDropFiles(HDROP hDrop);
-#endif
   afx_msg void OnColumnClick(NMHDR *pNotifyStruct, LRESULT *pLResult);
   afx_msg void OnUpdateNSCommand(CCmdUI *pCmdUI);  // Make entry unsupported (grayed out)
   afx_msg void OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);
@@ -899,6 +877,8 @@ private:
   void RefreshEntryFieldInGUI(CItemData &ci, CItemData::FieldType ft);
   void RefreshEntryPasswordInGUI(CItemData &ci);
   void RebuildGUI(const int iView = iBothViews);
+  void UpdateEntryinGUI(CItemData &ci);
+  StringX GetListViewItemText(CItemData &ci, const int &icolumn);
   
   static const struct UICommandTableEntry {
     UINT ID;
