@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -235,10 +235,8 @@ int DboxMain::RestoreSafe()
 
   m_core.SetCurFile(L"");    // Force a Save As...
   m_core.SetDBChanged(true); // So that the restored file will be saved
-#if !defined(POCKET_PC)
   m_titlebar.LoadString(IDS_UNTITLEDRESTORE);
   app.SetTooltipText(L"PasswordSafe");
-#endif
   ChangeOkUpdate();
   RefreshViews();
 
@@ -352,6 +350,7 @@ void DboxMain::OnOptions()
         dwExtendedStyle &= ~LVS_EX_GRIDLINES;
       }
       m_ctlItemList.SetExtendedStyle(dwExtendedStyle);
+      m_ctlItemList.UpdateRowHeight(true);
     }
 
     m_ctlItemTree.ActivateND(pOptionsPS->GetNotesAsTips());
