@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -12,7 +12,7 @@ class CPWPropertyPage : public CPropertyPage
 {
 public:
   CPWPropertyPage(UINT nID);
-  virtual ~CPWPropertyPage() {}
+  virtual ~CPWPropertyPage() {delete m_pToolTipCtrl;}
 
   // Following override to reset idle timeout on any event
   virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
@@ -23,6 +23,14 @@ public:
         PP_UPDATE_TIMES};
 
   DECLARE_DYNAMIC(CPWPropertyPage)
+protected:
+  void InitToolTip(int Flags = TTS_BALLOON | TTS_NOPREFIX, int delayTimeFactor = 1);
+  void AddTool(int DlgItemID, int ResID);
+  void ActivateToolTip();
+  void RelayToolTipEvent(MSG *pMsg);
+  void ShowHelp(const CString &topicFile);
+
+  CToolTipCtrl *m_pToolTipCtrl;
 };
 //-----------------------------------------------------------------------------
 // Local variables:
