@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -8,20 +8,36 @@
 
 #pragma once
 
+class DboxMain;
+
 class CPWPropertySheet : public CPropertySheet
 {
 public:
-  CPWPropertySheet(UINT nID, CWnd* pParent);
+  CPWPropertySheet(UINT nID, CWnd* pParent, const bool bLongPPs);
 
-  CPWPropertySheet(LPCTSTR pszCaption, CWnd* pParent);
+  CPWPropertySheet(LPCTSTR pszCaption, CWnd* pParent, const bool bLongPPs);
 
   // Following override to reset idle timeout on any event
   virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
   // Following override to stop accelerators interfering
   virtual INT_PTR DoModal();
+  virtual BOOL OnInitDialog();
 
+protected:
   DECLARE_DYNAMIC(CPWPropertySheet)
+
+  afx_msg void OnWindowPosChanging(WINDOWPOS *lpwndpos);
+  afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);
+
+  DECLARE_MESSAGE_MAP()
+  
+private:
+  // Used to determine if Tall will fit in OnInitDialog
+  DboxMain *m_pDbx;
+  bool m_bLongPPs;
+  bool m_bKeepHidden;
 };
+
 //-----------------------------------------------------------------------------
 // Local variables:
 // mode: c++
