@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -28,5 +28,5 @@ public:
 
   virtual void ReadList();    // reads from registry or config file
   virtual void WriteList();   // writes to registry or config file
-  bool IsMRUEmpty();
+  bool IsMRUEmpty() {return GetSize() == 0;}
 };
