@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -80,6 +80,8 @@ static char THIS_FILE[] = __FILE__;
 
 IMPLEMENT_DYNAMIC(CSystemTray, CWnd)
 
+static const size_t MAX_TTT_LEN = 40; // Max tooltip text length for systray TT
+
 UINT CSystemTray::m_nIDEvent = 4567;
 const UINT CSystemTray::m_nTaskbarCreatedMsg = ::RegisterWindowMessage(L"TaskbarCreated");
 
@@ -113,7 +115,7 @@ BOOL CSystemTray::Create(CWnd *pParent, UINT uCallbackMessage, LPCWSTR szToolTip
   // Make sure we avoid conflict with other messages
   ASSERT(uCallbackMessage >= WM_USER);
 
-  StringX ttt = PWSUtil::NormalizeTTT(szToolTip);
+  StringX ttt = PWSUtil::NormalizeTTT(szToolTip, MAX_TTT_LEN);
 
   // Create an invisible window
   m_bEnabled = CWnd::CreateEx(0, AfxRegisterWndClass(0), L"", WS_POPUP, 0,0,10,10, NULL, 0);
@@ -148,6 +150,13 @@ CSystemTray::~CSystemTray()
   ::DestroyIcon(m_hSavedIcon);
 }
 
+CWnd *CSystemTray::SetTarget(CWnd *pTarget)
+{
+  CWnd *pOldTarget = m_pTarget;
+  m_pTarget = pTarget;
+  return pOldTarget;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // CSystemTray icon manipulation
 
@@ -315,7 +324,7 @@ BOOL CSystemTray::SetTooltipText(LPCWSTR pszTip)
   if (!m_bEnabled)
     return FALSE;
 
-  StringX ttt = PWSUtil::NormalizeTTT(pszTip);
+  StringX ttt = PWSUtil::NormalizeTTT(pszTip, MAX_TTT_LEN);
   m_tnd.uFlags = NIF_TIP;
 #if (_MSC_VER >= 1400)
   wcsncpy_s(m_tnd.szTip, sizeof(m_tnd.szTip), ttt.c_str(), ttt.length());
