@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -65,9 +65,7 @@ public:
 
   virtual BOOL InitInstance();
   virtual int ExitInstance();
-#if !defined(POCKET_PC)
   virtual BOOL ProcessMessageFilter(int code, LPMSG lpMsg);
-#endif
 
   void EnableAccelerator() { m_bUseAccelerator = true; }
   void DisableAccelerator() { m_bUseAccelerator = false; }
@@ -101,6 +99,8 @@ public:
   DWORD GetBaseThreadID() {return m_nBaseThreadID;}
   void GetLanguageFiles();
   void SetLanguage();
+  CWnd *SetSystemTrayTarget(CWnd *pWnd)
+  {return m_pTrayIcon->SetTarget(pWnd);}
 
   void SetMinidumpUserStreams(const bool bOpen, const bool bRW, UserStream iStream = usAll);
 
