@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -23,6 +23,8 @@ public:
 
   enum {IDD = IDD_WZFINISH};
 
+  void DisableAbort();
+
 protected:
   BOOL OnInitDialog();
 
@@ -30,6 +32,7 @@ protected:
   //{{AFX_MSG(CWZFinish)
   afx_msg BOOL OnSetActive();
   afx_msg void OnHelp();
+  afx_msg void OnAbort();
   afx_msg void OnViewReport();
   LRESULT OnExecuteThreadEnded(WPARAM wParam, LPARAM );
   //}}AFX_MSG
@@ -38,6 +41,7 @@ protected:
 
 private:
   BOOL Execute();
+
   CReport *m_prpt;
   CWinThread *m_pExecuteThread; // Execute worker thread
   int DoExecuteThread(WZExecuteThreadParms * &pthdpms);
@@ -48,6 +52,7 @@ private:
   bool m_bInitDone, m_bInProgress, m_bComplete, m_bViewingReport;
   int m_status;
   int m_numProcessed;
+  WZExecuteThreadParms m_thdpms;
 };
 //-----------------------------------------------------------------------------
 // Local variables:
