@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -16,7 +16,7 @@ public:
   DECLARE_DYNAMIC(CWZPropertyPage)
 
   CWZPropertyPage(UINT nID, UINT nIDCaption = 0, const int nType = INVALID);
-  ~CWZPropertyPage() {}
+  ~CWZPropertyPage() {delete m_pToolTipCtrl;}
 
   // Following override to reset idle timeout on any event
   virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
@@ -25,10 +25,12 @@ public:
 
 protected:
   CWZPropertySheet *m_pWZPSH;
+  CToolTipCtrl *m_pToolTipCtrl;
 
   virtual BOOL OnInitDialog();
   virtual void DoDataExchange(CDataExchange* pDX);
   virtual BOOL OnSetActive();
+  void ShowHelp(const CString &topicFile);
 
   // Generated message map functions
   //{{AFX_MSG(CWZPropertyPage)
