@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -13,7 +13,7 @@
 #include "WZSelectDB.h"
 #include "WZPropertySheet.h"
 
-#include "PwFont.h"
+#include "Fonts.h"
 #include "TryAgainDlg.h"
 #include "SecString.h"
 #include "GeneralMsgBox.h"
@@ -136,15 +136,14 @@ END_MESSAGE_MAP()
 
 void CWZSelectDB::OnHelp()
 {
-  CString cs_HelpTopic = app.GetHelpFileName() + L"::/html/wzselectdb.html";
-  ::HtmlHelp(this->GetSafeHwnd(), (LPCWSTR)cs_HelpTopic, HH_DISPLAY_TOPIC, 0);
+  ShowHelp(L"::/html/wzselectdb.html");
 }
 
 BOOL CWZSelectDB::OnInitDialog()
 {
   CWZPropertyPage::OnInitDialog();
 
-  ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
 
   m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
 
@@ -238,7 +237,7 @@ BOOL CWZSelectDB::OnInitDialog()
   return FALSE;
 }
 
-HBRUSH CWZSelectDB::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
+HBRUSH CWZSelectDB::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
 {
   HBRUSH hbr = CWZPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);
 
@@ -375,6 +374,11 @@ LRESULT CWZSelectDB::OnWizardNext()
         // pws_os::FOpen(...) cannot create any missing directories
         std::wstring dir, cdrive, cdir, cfile, cextn;
         pws_os::splitpath(m_filespec.GetString(), cdrive, cdir, cfile, cextn);
+        //  If root directory - nothing to create
+        if (cdir == L"\\")          
+          break;
+
+        // Try and create necessary directories
         dir = cdrive + cdir;
         int rc = ::SHCreateDirectoryEx(NULL, dir.c_str(), NULL);
         switch (rc) {
@@ -404,7 +408,7 @@ LRESULT CWZSelectDB::OnWizardNext()
   // Check that this file isn't already open
   const StringX sx_Filename1(m_pWZPSH->WZPSHGetCurFile());
   const StringX sx_Filename2 = m_filespec.GetString();
-  const StringX sx_passkey = StringX(m_passkey);
+  const StringX sx_passkey = (LPCWSTR)m_passkey;
   const bool bOtherIsDB = nID == ID_MENUITEM_COMPARE ||
                           nID == ID_MENUITEM_MERGE   ||
                           nID == ID_MENUITEM_SYNCHRONIZE;
@@ -428,15 +432,13 @@ LRESULT CWZSelectDB::OnWizardNext()
 
 bool CWZSelectDB::ProcessPhrase(const StringX &filename, const StringX &passkey)
 {
-  PWScore *pothercore = new PWScore;
-  if (m_pWZPSH->WZPSHCheckPasskey(filename, passkey, pothercore) == PWScore::SUCCESS) {
+  PWScore tmpcore;
+  if (m_pWZPSH->WZPSHCheckPasskey(filename, passkey,
+                                  &tmpcore) == PWScore::SUCCESS) {
     m_tries = 0;
-    delete [] pothercore;
     return true;
   }
 
-  delete [] pothercore;
-
   if (m_tries >= 2) {
     CTryAgainDlg errorDlg(this);
 
@@ -500,7 +502,8 @@ void CWZSelectDB::OnOpenFileBrowser()
 
   CString cs_text(MAKEINTRESOURCE(uimsgid));
 
-  std::wstring ExportFileName = PWSUtil::GetNewFileName(m_pWZPSH->WZPSHGetCurFile().c_str(), L"txt");
+  std::wstring ExportFileName = PWSUtil::GetNewFileName(m_pWZPSH->WZPSHGetCurFile().c_str(),
+                                                        CString::PCXSTR(cs_suffix));
   CPWFileDialog fd(bTYPE_OPEN, cs_suffix, uimsgid != IDS_CHOOSEDATABASE ? ExportFileName.c_str() : NULL, 
                    dwflags, cs_filter, this);
 
