@@ -30,7 +30,7 @@
 #define IDD_PS_DISPLAY                  138
 #define IDD_PS_MISC                     139
 #define IDD_PS_PASSWORDHISTORY          140
-#define IDD_PS_PASSWORDPOLICY           141
+#define IDD_PASSWORDPOLICY              141
 #define IDD_PS_SECURITY                 142
 #define IDD_PS_SYSTEM                   143
 #define IDD_QUERYSETDEF                 144
@@ -70,7 +70,6 @@
 #define IDI_LOCKEDICON                  181
 #define IDI_MBASKICON                   182
 #define IDI_MBINFOICON                  183
-#define IDI_MBINFOICON                  183
 #define IDI_MBSTOPICON                  184
 #define IDI_MBWARNICON                  185
 #define IDI_ALIAS                       186
@@ -90,6 +89,7 @@
 #define IDI_TRAY_YELLOW                 208
 #define IDI_UNLOCKEDICON                209
 #define IDR_POPLANGUAGES                210
+#define IDR_POPCOPYALLTOORIGINAL        211
 #define IDB_CLOGO                       220
 #define IDB_CLOGO_SMALL                 221
 #define IDB_PSLOGO                      223
@@ -98,7 +98,7 @@
 #define IDB_ABASE                       231
 #define IDB_ABASE_EXPIRED               232
 #define IDB_ABASE_WARNEXPIRED           233
-#define IDB_NODE                        234
+#define IDB_GROUP                       234
 #define IDB_NORMAL                      235
 #define IDB_NORMAL_EXPIRED              236
 #define IDB_NORMAL_WARNEXPIRED          237
@@ -345,10 +345,23 @@
 #define IDD_PS_BACKUP_SHORT             505
 #define IDD_PS_DISPLAY_SHORT            506
 #define IDD_PS_MISC_SHORT               507
-#define IDD_PS_PASSWORDPOLICY_SHORT     508
+#define IDD_PASSWORDPOLICY_SHORT        508
 #define IDD_PS_SHORTCUTS_SHORT          509
 #define IDD_PS_SYSTEM_SHORT             510
 #define IDD_INPUTBOX                    511
+#define IDB_CLEARFILTERS_CLASSIC        512
+#define IDB_CLEARFILTERS_NEW            513
+#define IDB_CLEARFILTERS_NEW_D          514
+#define IDD_MANAGEPASSWORDPOLICIES      515
+#define IDB_PSWD_POLICIES_CLASSIC       516
+#define IDB_PSWD_POLICIES_NEW           517
+#define IDB_PSWD_POLICIES_NEW_D         518
+#define IDB_COPYTOCLIPBOARD_CLASSIC     519
+#define IDB_COPYTOCLIPBOARD_NEW         520
+#define IDR_POPCOMPAREENTRIES           521
+#define IDD_SHOW_COMPARE                522
+#define IDD_COMPARE_WITH                523
+#define IDB_EMPTY_GROUP                 524
 #define IDC_APPVERSION                  1000
 #define IDC_VERSION                     1001
 #define IDC_CREATE_DB                   1003
@@ -368,12 +381,11 @@
 #define IDC_DELITEM                     1017
 #define IDC_VERIFY                      1018
 #define IDC_CONFIRMDELETE               1019
-#define IDC_NOTESWW                     1020
 #define IDC_CONFIRMCOPY                 1021
 #define IDC_TRYAGAIN                    1022
 #define IDC_QUIT                        1023
 #define IDC_OPEN                        1024
-#define IDC_RANDOM                      1025
+#define IDC_GENERATEPASSWORD            1025
 #define IDC_LOCKONMINIMIZE              1026
 #define IDC_DEFPWSHOWINEDIT             1028
 #define IDC_USEDEFUSER                  1029
@@ -495,6 +507,7 @@
 #define IDC_SETMAXPWHISTORY             1186
 #define IDC_STATIC_UPDATEPWHISTORY      1187
 #define IDC_REGDEL                      1188
+#define IDC_SETMAXPWHISTORY2            1188
 #define IDC_DEFNOTESSHOWINEDIT          1190
 #define IDC_DATABASENAME                1191
 #define IDC_DATABASEFORMAT              1192
@@ -649,10 +662,7 @@
 #define IDC_VKB                         1372
 #define IDC_SETSAMPLETEXT               1373
 #define IDC_RESETFONT                   1374
-#define IDC_USEDEFAULTPWPOLICY          1375
 #define IDC_ENTRYPWPOLICY               1376
-#define IDC_RESETPWPOLICY               1377
-#define IDC_DCA_DEFAULT                 1378
 #define IDC_LAUNCH                      1379
 #define IDC_SENDEMAIL                   1380
 #define IDC_STATIC_PWH_ADD              1381
@@ -723,7 +733,6 @@
 #define IDC_STATIC_DEFAUTOTYPE          1447
 #define IDC_STATIC_PSWDLENGTH           1448
 #define IDC_SHIFT_DOUBLE_CLICK_ACTION   1449
-#define IDC_SHIFT_DCA_DEFAULT           1450
 #define IDC_STATIC_PREFERENCES          1453
 #define IDC_STATIC_DB_PREFS_RO_WARNING  1454
 #define IDC_DATABASE_NAME               1455
@@ -732,14 +741,34 @@
 #define IDC_CHANGE_NAME                 1458
 #define IDC_INPUTBOX_TEXT               1459
 #define IDC_INPUTBOX_CHAR_COUNT         1460
+#define IDC_EDIT                        1461
+#define IDC_LIST_POLICYENTRIES          1462
+#define IDC_POLICYLIST                  1463
+#define IDC_POLICYNAME                  1464
+#define IDC_USEPWPOLICYNAME             1465
+#define IDC_POLICYPROPERTIES            1466
+#define IDC_POLICYENTRIES               1467
+#define IDC_STATIC_POLICYDETAILS        1468
+#define IDC_STATIC_POLICYENTRIES        1469
+#define IDC_USENAMED_POLICY             1470
+#define IDC_STATIC_NAMEDPOLICY          1471
+#define IDC_POLICYNAMES_HEADER          1472
+#define IDC_POLICYENTRIES_HEADER        1473
+#define IDC_STATIC_COPYPSWD             1474
+#define IDC_UNDO                        1475
+#define IDC_REDO                        1476
+#define IDC_CLEARPWHISTORY              1477
+#define IDC_UPDATEPROTECTEDPWH          1478
+#define IDC_SHOW_IDENTICAL_FIELDS       1479
+#define IDC_ABORT                       1480
 
 // Next default values for new objects
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
-#define _APS_NEXT_RESOURCE_VALUE        512
+#define _APS_NEXT_RESOURCE_VALUE        525
 #define _APS_NEXT_COMMAND_VALUE         30001
-#define _APS_NEXT_CONTROL_VALUE         1461
+#define _APS_NEXT_CONTROL_VALUE         1481
 #define _APS_NEXT_SYMED_VALUE           166
 #endif
 #endif
