@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -7,7 +7,7 @@
  */
 
 /** \file about.cpp
-* 
+*
 */
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
@@ -24,76 +24,13 @@
 #include <wx/statline.h>
 
 #include "AdvancedSelectionDlg.h"
+#include "SelectionCriteria.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/msvcrt.h>
 #endif
 
-CItemData::FieldType subgroups[] = {  CItemData::GROUP,
-                                      CItemData::GROUPTITLE,
-                                      CItemData::NOTES,
-                                      CItemData::TITLE,
-                                      CItemData::URL,
-                                      CItemData::USER
-                                  } ;
-
-struct _subgroupFunctions subgroupFunctions[] = { {wxT("equals"),              PWSMatch::MR_EQUALS},
-                                                  {wxT("does not equal"),      PWSMatch::MR_NOTEQUAL},
-                                                  {wxT("begins with"),         PWSMatch::MR_BEGINS},
-                                                  {wxT("does not begin with"), PWSMatch::MR_NOTBEGIN},
-                                                  {wxT("ends with"),           PWSMatch::MR_ENDS},
-                                                  {wxT("does not end with"),   PWSMatch::MR_NOTEND},
-                                                  {wxT("contains"),            PWSMatch::MR_CONTAINS},
-                                                  {wxT("does not contain"),    PWSMatch::MR_NOTCONTAIN} } ;
-
-CItemData::FieldType selectableFields[] = { CItemData::GROUP,
-                                        CItemData::TITLE,
-                                        CItemData::USER,
-                                        CItemData::NOTES,
-                                        CItemData::PASSWORD,
-                                        CItemData::URL,
-                                        CItemData::AUTOTYPE,
-                                        CItemData::PWHIST,
-                                        CItemData::RUNCMD,
-                                        CItemData::EMAIL
-                                      };
-
-
-
-wxString GetSelectableFieldName(CItemData::FieldType ft) {
-  return towxstring(CItemData::FieldName(ft));
-}
-
-size_t GetNumFieldsSelectable() {
-  return WXSIZEOF(selectableFields);
-}
-
-CItemData::FieldType GetSelectableField(size_t idx) {
-  wxASSERT_MSG(idx < GetNumFieldsSelectable(), wxT("Invalid index for GetSelectableField"));
-  return selectableFields[idx];
-}
-
-wxString SelectionCriteria::GetGroupSelectionDescription() const
-{
-  if (!m_fUseSubgroups)
-    return _("All entries");
-  else
-    return wxString(_("Entries whose ")) << GetSelectableFieldName(subgroups[m_subgroupObject]) << wxT(' ')
-            << subgroupFunctions[m_subgroupFunction].name << wxT(" \"") << m_subgroupText
-            << wxT("\" [") << (m_fCaseSensitive? wxT("") : _("not ")) << _("case-sensitive]");
-}
-
-//returns true if all fields have been selected
-bool SelectionCriteria::GetFieldSelection(wxArrayString& selectedFields, wxArrayString& unselectedFields)
-{
-  for (size_t idx = 0; idx < GetNumFieldsSelectable(); ++idx) {
-    if (m_bsFields.test(selectableFields[idx]))
-      selectedFields.Add(GetSelectableFieldName(selectableFields[idx]));
-    else
-      unselectedFields.Add(GetSelectableFieldName(selectableFields[idx]));
-  }
-  return m_bsFields.count() == NumberOf(selectableFields);
-}
+void EnableSizerElements(wxSizer* sizer, wxWindow* ignore, bool enable);
 
 ////////////////////////////////////////////////////////////////////////////
 // AdvancedSelectionPanel implementation
@@ -110,8 +47,8 @@ BEGIN_EVENT_TABLE( AdvancedSelectionPanel, wxPanel )
 END_EVENT_TABLE()
 
 
-AdvancedSelectionPanel::AdvancedSelectionPanel(wxWindow* parentWnd, 
-                                               const SelectionCriteria& existingCriteria,
+AdvancedSelectionPanel::AdvancedSelectionPanel(wxWindow* parentWnd,
+                                               SelectionCriteria* existingCriteria,
                                                bool autoValidate):
                                                   m_criteria(existingCriteria),
                                                   m_autoValidate(autoValidate)
@@ -123,72 +60,76 @@ AdvancedSelectionPanel::AdvancedSelectionPanel(wxWindow* parentWnd,
 void AdvancedSelectionPanel::CreateControls(wxWindow* parentWnd)
 {
   wxPanel::Create(parentWnd);
-  
+
   wxBoxSizer* dlgSizer = new wxBoxSizer(wxVERTICAL);
-  
+
   //Subset entries
   {
     wxStaticBoxSizer* sizer = new wxStaticBoxSizer(wxVERTICAL, this);
 
-    wxCheckBox* check = new wxCheckBox(this, wxID_ANY, wxT("&Restrict to a subset of entries:"));
-    check->SetValidator(wxGenericValidator(&m_criteria.m_fUseSubgroups));
+    wxCheckBox* check = new wxCheckBox(this, wxID_ANY, _("&Restrict to a subset of entries:"));
+    check->SetValidator(wxGenericValidator(&m_criteria->m_fUseSubgroups));
     sizer->Add(check, wxSizerFlags().Border());
+    check->Connect(wxEVT_COMMAND_CHECKBOX_CLICKED,
+                    wxCommandEventHandler(AdvancedSelectionPanel::OnRestrictSearchItems));
 
     wxBoxSizer* hbox = new wxBoxSizer(wxHORIZONTAL);
-    hbox->Add(new wxStaticText(this, wxID_ANY, wxT("&Where")), wxSizerFlags(0));
+    hbox->Add(new wxStaticText(this, wxID_ANY, _("&Where")), wxSizerFlags(0));
     hbox->AddSpacer(ColSeparation);
-    
+
     wxComboBox* comboSubgroup = new wxComboBox(this, wxID_ANY);
-    for (size_t idx = 0 ; idx < NumberOf(subgroups); ++idx) comboSubgroup->AppendString(GetSelectableFieldName(subgroups[idx]));
-    comboSubgroup->SetValidator(wxGenericValidator(&m_criteria.m_subgroupObject));
+    for (size_t idx = 0 ; idx < SelectionCriteria::GetNumSubgroups(); ++idx) comboSubgroup->AppendString(SelectionCriteria::GetSelectableFieldName(SelectionCriteria::GetSubgroup(idx)));
+    comboSubgroup->SetValidator(wxGenericValidator(&m_criteria->m_subgroupObject));
     hbox->Add(comboSubgroup, wxSizerFlags(1).Expand());
-    
+
     hbox->AddSpacer(ColSeparation);
-    
+
     wxComboBox* comboFunctions = new wxComboBox(this, wxID_ANY);
-    for( size_t idx = 0; idx < NumberOf(subgroupFunctions); ++idx) comboFunctions->AppendString(subgroupFunctions[idx].name);
-    comboFunctions->SetValidator(wxGenericValidator(&m_criteria.m_subgroupFunction));
+    for( size_t idx = 0; idx < SelectionCriteria::GetNumSubgroupFunctions(); ++idx) comboFunctions->AppendString(SelectionCriteria::GetSubgroupFunctionName(idx));
+    comboFunctions->SetValidator(wxGenericValidator(&m_criteria->m_subgroupFunction));
     hbox->Add(comboFunctions, wxSizerFlags(1).Expand());
-    
+
     sizer->Add(hbox, wxSizerFlags().Border().Expand());
 
-    sizer->Add( new wxStaticText(this, wxID_ANY, wxT("the &following text:")), wxSizerFlags().Border());
+    sizer->Add( new wxStaticText(this, wxID_ANY, _("the &following text:")), wxSizerFlags().Border());
 
     wxTextCtrl* txtCtrl = new wxTextCtrl(this, wxID_ANY, _("*"), wxDefaultPosition, wxSize(200, -1));
-    txtCtrl->SetValidator(wxGenericValidator(&m_criteria.m_subgroupText));
+    txtCtrl->SetValidator(wxGenericValidator(&m_criteria->m_subgroupText));
     sizer->Add(txtCtrl, wxSizerFlags().Border().Expand().FixedMinSize());
 
-    wxCheckBox* checkCaseSensitivity = new wxCheckBox(this, wxID_ANY, wxT("&Case Sensitive"));
-    checkCaseSensitivity->SetValidator(wxGenericValidator(&m_criteria.m_fCaseSensitive));
+    wxCheckBox* checkCaseSensitivity = new wxCheckBox(this, wxID_ANY, _("&Case Sensitive"));
+    checkCaseSensitivity->SetValidator(wxGenericValidator(&m_criteria->m_fCaseSensitive));
     sizer->Add( checkCaseSensitivity, wxSizerFlags().Border() );
-    
+
     dlgSizer->Add(sizer, wxSizerFlags().Border(wxLEFT|wxRIGHT, SideMargin).Expand());
+
+    EnableSizerElements(sizer, check, m_criteria->HasSubgroupRestriction());
   }
 
   if (ShowFieldSelection()) {
-    
+
     dlgSizer->AddSpacer(RowSeparation);
-    
+
     {
       wxFlexGridSizer* grid = new wxFlexGridSizer(3, RowSeparation, ColSeparation);
-      
+
       //first and third columns are growable
-      grid->AddGrowableCol(0, 1);  
+      grid->AddGrowableCol(0, 1);
       grid->AddGrowableCol(2, 1);
       grid->AddGrowableRow(1, 1);
       grid->SetFlexibleDirection(wxBOTH);
-      
+
       //first row is labels, with a spacer in between
-      grid->Add(new wxStaticText(this, wxID_ANY, wxT("&Available Fields:")));
+      grid->Add(new wxStaticText(this, wxID_ANY, _("&Available Fields:")));
       grid->AddSpacer(0);
-      grid->Add(new wxStaticText(this, wxID_ANY, wxT("&Selected Fields:")));
-      
+      grid->Add(new wxStaticText(this, wxID_ANY, _("&Selected Fields:")));
+
       //second row is the listboxes, with buttons in between
 
-      wxListBox* lbAvailable = new wxListBox(this, ID_LB_AVAILABLE_FIELDS, wxDefaultPosition, 
+      wxListBox* lbAvailable = new wxListBox(this, ID_LB_AVAILABLE_FIELDS, wxDefaultPosition,
                                              wxDefaultSize, 0, NULL, wxLB_EXTENDED);
       grid->Add(lbAvailable, wxSizerFlags().Expand());
-      
+
       wxBoxSizer* buttonBox = new wxBoxSizer(wxVERTICAL);
       buttonBox->AddStretchSpacer();
       buttonBox->Add( new wxButton(this, ID_SELECT_SOME, wxT(">")) );
@@ -199,24 +140,24 @@ void AdvancedSelectionPanel::CreateControls(wxWindow* parentWnd)
       buttonBox->AddSpacer(RowSeparation);
       buttonBox->Add( new wxButton(this, ID_REMOVE_ALL, wxT("<<")) );
       buttonBox->AddStretchSpacer();
-      
+
       grid->Add(buttonBox, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
 
-      wxListBox* lbSelected = new wxListBox(this, ID_LB_SELECTED_FIELDS, wxDefaultPosition, 
+      wxListBox* lbSelected = new wxListBox(this, ID_LB_SELECTED_FIELDS, wxDefaultPosition,
                                             wxDefaultSize, 0, NULL, wxLB_EXTENDED);
       grid->Add(lbSelected, wxSizerFlags().Expand());
 
       dlgSizer->Add(grid, wxSizerFlags(1).Expand().Border(wxLEFT | wxRIGHT, SideMargin));
-      
+
       //add all the field names to both listboxes to size the dialog/wizard page correctly
       //These are anyway removed in TransferDataToWindow below before doing anything else
-      for (size_t idx=0; idx < WXSIZEOF(selectableFields); ++idx) {
-        lbAvailable->Append(GetSelectableFieldName(selectableFields[idx]), reinterpret_cast<void *>(idx));
-        lbSelected->Append(GetSelectableFieldName(selectableFields[idx]), reinterpret_cast<void *>(idx));
-      }      
+      for (size_t idx=0; idx < SelectionCriteria::GetNumFieldsSelectable(); ++idx) {
+        lbAvailable->Append(SelectionCriteria::GetSelectableFieldName(SelectionCriteria::GetSelectableField(idx)), reinterpret_cast<void *>(idx));
+        lbSelected->Append(SelectionCriteria::GetSelectableFieldName(SelectionCriteria::GetSelectableField(idx)), reinterpret_cast<void *>(idx));
+      }
     }
   }
-  
+
   SetSizerAndFit(dlgSizer);
 }
 
@@ -224,25 +165,25 @@ bool AdvancedSelectionPanel::TransferDataToWindow()
 {
   if (wxPanel::TransferDataToWindow()) {
     if (ShowFieldSelection()) {
-
+      // Temporary hack until I can write a proper validator for SelectionCriteria class
+      // which would set its dirty flag automatically
+      const bool criteriaChanged = (*m_criteria != SelectionCriteria());
       wxListBox* lbAvailable  = wxDynamicCast(FindWindow(ID_LB_AVAILABLE_FIELDS), wxListBox);
-      lbAvailable->Clear();
-
-      for (size_t idx = 0; idx < NumberOf(selectableFields); ++idx) {
-        if (!m_criteria.m_bsFields.test(selectableFields[idx]))
-            lbAvailable->Append(GetSelectableFieldName(selectableFields[idx]), reinterpret_cast<void *>(idx));
-      }
-
       wxListBox* lbSelected  = wxDynamicCast(FindWindow(ID_LB_SELECTED_FIELDS), wxListBox);
+      lbAvailable->Clear();
       lbSelected->Clear();
 
-      for (size_t idx=0; idx < NumberOf(selectableFields); ++idx) {
-        if (m_criteria.m_bsFields.test(selectableFields[idx])) {
-          if (IsMandatoryField(selectableFields[idx]))
-            lbSelected->Append(GetSelectableFieldName(selectableFields[idx]) + _(" [Mandatory Field]"),
-                                     reinterpret_cast<void *>(idx));
-          else
-            lbSelected->Append(GetSelectableFieldName(selectableFields[idx]), reinterpret_cast<void *>(idx));
+      for (size_t idx = 0; idx < SelectionCriteria::GetNumFieldsSelectable(); ++idx) {
+        const CItemData::FieldType ft = SelectionCriteria::GetSelectableField(idx);
+        if (IsUsableField(ft)) {
+          if ( (criteriaChanged && m_criteria->IsFieldSelected(ft)) ||
+                              (!criteriaChanged && IsPreselectedField(ft)) ) {
+            const wxString title = SelectionCriteria::GetSelectableFieldName(ft) + (IsMandatoryField(ft)? _(" [Mandatory Field]"): wxEmptyString);
+            lbSelected->Append(title, reinterpret_cast<void *>(idx));
+          }
+          else {
+            lbAvailable->Append(SelectionCriteria::GetSelectableFieldName(ft), reinterpret_cast<void *>(idx));
+          }
         }
       }
     }
@@ -257,8 +198,8 @@ bool AdvancedSelectionPanel::DoValidation()
     wxListBox* lbSelected  = wxDynamicCast(FindWindow(ID_LB_SELECTED_FIELDS), wxListBox);
     wxASSERT(lbSelected);
     if (lbSelected->GetCount() == 0) {
-      wxMessageBox(wxString(_("You must select some of the fields to ")) << GetTaskWord(), 
-                        wxT("No fields selected"), wxOK|wxICON_INFORMATION, this);
+      wxMessageBox(wxString(_("You must select some of the fields to ")) << GetTaskWord(),
+                        _("No fields selected"), wxOK|wxICON_INFORMATION, this);
       return false;
     }
   }
@@ -277,13 +218,13 @@ bool AdvancedSelectionPanel::TransferDataFromWindow()
       wxListBox* lbSelected  = wxDynamicCast(FindWindow(ID_LB_SELECTED_FIELDS), wxListBox);
       wxASSERT(lbSelected);
 
-      //reset the selected field bits 
-      m_criteria.m_bsFields.reset();
+      //reset the selected field bits
+      m_criteria->m_bsFields.reset();
       const size_t count = lbSelected->GetCount();
-      
+
       for (size_t idx = 0; idx < count; ++idx) {
           const size_t which = reinterpret_cast<size_t>(lbSelected->GetClientData(static_cast<unsigned int>(idx)));
-          m_criteria.m_bsFields.set(selectableFields[which], true);
+          m_criteria->SelectField(SelectionCriteria::GetSelectableField(which));
       }
     }
     return true;
@@ -295,7 +236,7 @@ void AdvancedSelectionPanel::OnSelectSome( wxCommandEvent& /* evt */ )
 {
   wxListBox* lbAvailable = wxDynamicCast(FindWindow(ID_LB_AVAILABLE_FIELDS), wxListBox);
   wxListBox* lbSelected  = wxDynamicCast(FindWindow(ID_LB_SELECTED_FIELDS), wxListBox);
-  
+
   wxASSERT(lbAvailable);
   wxASSERT(lbSelected);
 
@@ -303,9 +244,9 @@ void AdvancedSelectionPanel::OnSelectSome( wxCommandEvent& /* evt */ )
   if (lbAvailable->GetSelections(aSelected)) {
     for (size_t idx = 0; idx < aSelected.GetCount(); ++idx) {
       size_t which = reinterpret_cast<size_t>(lbAvailable->GetClientData(static_cast<unsigned int>(aSelected[idx] - idx)));
-      wxASSERT(which < NumberOf(selectableFields));
+      wxASSERT(which < SelectionCriteria::GetNumFieldsSelectable());
       lbAvailable->Delete(static_cast<unsigned int>(aSelected[idx] - idx));
-      lbSelected->Append(GetSelectableFieldName(selectableFields[which]), reinterpret_cast<void *>(which));
+      lbSelected->Append(SelectionCriteria::GetSelectableFieldName(SelectionCriteria::GetSelectableField(which)), reinterpret_cast<void *>(which));
     }
   }
 }
@@ -314,14 +255,14 @@ void AdvancedSelectionPanel::OnSelectAll( wxCommandEvent& /* evt */ )
 {
   wxListBox* lbAvailable = wxDynamicCast(FindWindow(ID_LB_AVAILABLE_FIELDS), wxListBox);
   wxListBox* lbSelected  = wxDynamicCast(FindWindow(ID_LB_SELECTED_FIELDS), wxListBox);
-  
+
   wxASSERT(lbAvailable);
   wxASSERT(lbSelected);
 
   while (lbAvailable->GetCount()) {
       size_t which = reinterpret_cast<size_t>(lbAvailable->GetClientData(0));
       lbAvailable->Delete(0);
-      lbSelected->Append(GetSelectableFieldName(selectableFields[which]), reinterpret_cast<void *>(which));
+      lbSelected->Append(SelectionCriteria::GetSelectableFieldName(SelectionCriteria::GetSelectableField(which)), reinterpret_cast<void *>(which));
   }
 }
 
@@ -329,7 +270,7 @@ void AdvancedSelectionPanel::OnRemoveSome( wxCommandEvent& /* evt */ )
 {
   wxListBox* lbAvailable = wxDynamicCast(FindWindow(ID_LB_AVAILABLE_FIELDS), wxListBox);
   wxListBox* lbSelected  = wxDynamicCast(FindWindow(ID_LB_SELECTED_FIELDS), wxListBox);
-  
+
   wxASSERT(lbAvailable);
   wxASSERT(lbSelected);
 
@@ -337,10 +278,10 @@ void AdvancedSelectionPanel::OnRemoveSome( wxCommandEvent& /* evt */ )
   if (lbSelected->GetSelections(aSelected)) {
     for (size_t idx = 0, nRemoved = 0; idx < aSelected.GetCount(); ++idx) {
       size_t which = reinterpret_cast<size_t>(lbSelected->GetClientData(static_cast<unsigned int>(aSelected[idx] - nRemoved)));
-      wxASSERT(which < NumberOf(selectableFields));
-      if (!IsMandatoryField(selectableFields[which])) {
+      wxASSERT(which < SelectionCriteria::GetNumFieldsSelectable());
+      if (!IsMandatoryField(SelectionCriteria::GetSelectableField(which))) {
         lbSelected->Delete(static_cast<unsigned int>(aSelected[idx] - nRemoved++));
-        lbAvailable->Append(GetSelectableFieldName(selectableFields[which]), reinterpret_cast<void *>(which));
+        lbAvailable->Append(SelectionCriteria::GetSelectableFieldName(SelectionCriteria::GetSelectableField(which)), reinterpret_cast<void *>(which));
       }
     }
   }
@@ -350,15 +291,15 @@ void AdvancedSelectionPanel::OnRemoveAll( wxCommandEvent& /* evt */ )
 {
   wxListBox* lbAvailable = wxDynamicCast(FindWindow(ID_LB_AVAILABLE_FIELDS), wxListBox);
   wxListBox* lbSelected  = wxDynamicCast(FindWindow(ID_LB_SELECTED_FIELDS), wxListBox);
-  
+
   wxASSERT(lbAvailable);
   wxASSERT(lbSelected);
 
   for(size_t itemsLeft = lbSelected->GetCount(), idx = 0; idx < itemsLeft; ) {
       size_t which = reinterpret_cast<size_t>(lbSelected->GetClientData(reinterpret_cast<unsigned int &>(idx)));
-      if (!IsMandatoryField(selectableFields[which])) {
+      if (!IsMandatoryField(SelectionCriteria::GetSelectableField(which))) {
         lbSelected->Delete(reinterpret_cast<unsigned int &>(idx));
-        lbAvailable->Append(GetSelectableFieldName(selectableFields[which]), reinterpret_cast<void *>(which));
+        lbAvailable->Append(SelectionCriteria::GetSelectableFieldName(SelectionCriteria::GetSelectableField(which)), reinterpret_cast<void *>(which));
         --itemsLeft;
       }
       else
@@ -366,3 +307,29 @@ void AdvancedSelectionPanel::OnRemoveAll( wxCommandEvent& /* evt */ )
   }
 }
 
+/*
+ * Recursively enables/disables all sizer elements.  The <ignore> window
+ * is not disabled
+ */
+void EnableSizerElements(wxSizer* sizer, wxWindow* ignore, bool enable)
+{
+  wxCHECK_RET(sizer, wxT("Null sizer passed to EnableSizerElements"));
+
+  wxSizerItemList& items = sizer->GetChildren();
+  for (wxSizerItemList::iterator itr = items.begin(); itr != items.end(); ++itr) {
+    wxSizerItem* item = *itr;
+    if (item->IsWindow() && item->GetWindow() != ignore)
+      item->GetWindow()->Enable(enable);
+    else if (item->IsSizer())
+      EnableSizerElements(item->GetSizer(), ignore, enable);
+  }
+}
+
+void AdvancedSelectionPanel::OnRestrictSearchItems(wxCommandEvent& evt)
+{
+  wxWindow* checkbox = wxDynamicCast(evt.GetEventObject(), wxWindow);
+  wxCHECK_RET(checkbox, wxT("Could not get checkbox from check event object"));
+  wxSizer* sizer = checkbox->GetContainingSizer();
+  wxCHECK_RET(sizer, wxT("Could not get the sizer owning the checkbox"));
+  EnableSizerElements(sizer, checkbox, evt.IsChecked());
+}
