@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -7,7 +7,7 @@
  */
 
 /** \file optionspropsheet.cpp
-* 
+*
 */
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
@@ -36,10 +36,16 @@
 #include "core/PWSprefs.h"
 #include "core/Util.h" // for datetime string
 #include "core/PWSAuxParse.h" // for DEFAULT_AUTOTYPE
+#include "./wxutils.h"
+#include "./pwsmenushortcuts.h"
+
+#ifdef __WXMSW__
+#include <wx/msw/msvcrt.h>
+#endif
+
 ////@begin XPM images
 ////@end XPM images
 
-
 /*!
  * COptions type definition
  */
@@ -56,59 +62,39 @@ EVT_BUTTON( wxID_OK, COptions::OnOk )
 
 ////@begin COptions event table entries
   EVT_CHECKBOX( ID_CHECKBOX11, COptions::OnBackupB4SaveClick )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON4, COptions::OnBuPrefix )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON5, COptions::OnBuPrefix )
-
   EVT_COMBOBOX( ID_COMBOBOX2, COptions::OnSuffixCBSet )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON6, COptions::OnBuDirRB )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON7, COptions::OnBuDirRB )
-
   EVT_BUTTON( ID_BUTTON, COptions::OnBuDirBrowseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX13, COptions::OnShowUsernameInTreeCB )
-
   EVT_CHECKBOX( ID_CHECKBOX19, COptions::OnPreExpiryWarnClick )
-
   EVT_CHECKBOX( ID_CHECKBOX24, COptions::OnUseDefaultUserClick )
-
   EVT_BUTTON( ID_BUTTON8, COptions::OnBrowseLocationClick )
-
   EVT_CHECKBOX( ID_CHECKBOX3, COptions::OnPwPolUseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX4, COptions::OnPwPolUseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX5, COptions::OnPwPolUseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX6, COptions::OnPwPolUseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX7, COptions::OnPwPolUseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX8, COptions::OnPwPolUseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX9, COptions::OnPwPolUseClick )
-
   EVT_CHECKBOX( ID_CHECKBOX26, COptions::OnPWHistSaveClick )
-
   EVT_RADIOBUTTON( ID_PWHISTNOCHANGE, COptions::OnPWHistRB )
-
   EVT_RADIOBUTTON( ID_PWHISTSTOP, COptions::OnPWHistRB )
-
   EVT_RADIOBUTTON( ID_PWHISTSTART, COptions::OnPWHistRB )
-
   EVT_RADIOBUTTON( ID_PWHISTSETMAX, COptions::OnPWHistRB )
-
   EVT_BUTTON( ID_PWHISTNOCHANGE, COptions::OnPWHistApply )
-
   EVT_CHECKBOX( ID_CHECKBOX29, COptions::OnLockOnIdleClick )
-
   EVT_CHECKBOX( ID_CHECKBOX30, COptions::OnUseSystrayClick )
-
 ////@end COptions event table entries
 
+  EVT_BOOKCTRL_PAGE_CHANGING(wxID_ANY, COptions::OnPageChanging)
+  EVT_BOOKCTRL_PAGE_CHANGING(wxID_ANY, COptions::OnPageChanging)
+  EVT_SPINCTRL(ID_SPINCTRL5, COptions::OnAtLeastChars)
+  EVT_SPINCTRL(ID_SPINCTRL6, COptions::OnAtLeastChars)
+  EVT_SPINCTRL(ID_SPINCTRL7, COptions::OnAtLeastChars)
+  EVT_SPINCTRL(ID_SPINCTRL8, COptions::OnAtLeastChars)
 END_EVENT_TABLE()
 
 const wxChar *BUSuffix[] = {
@@ -122,7 +108,7 @@ enum {NO_SFX, TS_SFX, INC_SFX}; // For backup file suffix name
 // Following in enum order (see PWSprefs.h)
 const wxChar *DCAStrings[] = {
   _("Copy password to clipboard"),
-  _("View/Edit selected entry"),
+  _("Edit/View selected entry"),
   _("Autotype"),
   _("Browse to URL"),
   _("Copy notes to clipboard"),
@@ -209,6 +195,7 @@ void COptions::Init()
   m_preexpirywarnCB = NULL;
   m_preexpirywarndaysSB = NULL;
   m_DCACB = NULL;
+  m_SDCACB = NULL;
   m_defusernameTXT = NULL;
   m_defusernameLBL = NULL;
   m_pwMinsGSzr = NULL;
@@ -243,7 +230,7 @@ void COptions::Init()
  */
 
 void COptions::CreateControls()
-{    
+{
 
   wxPanel* itemPanel2 = new wxPanel( GetBookCtrl(), ID_PANEL, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
   wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
@@ -424,6 +411,14 @@ void COptions::CreateControls()
   m_DCACB = new wxComboBox( itemPanel44, ID_COMBOBOX3, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_DCACBStrings, wxCB_READONLY );
   itemBoxSizer49->Add(m_DCACB, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
+  wxBoxSizer* itemBoxSizer49a = new wxBoxSizer(wxHORIZONTAL);
+  itemBoxSizer45->Add(itemBoxSizer49a, 0, wxGROW|wxALL, 0);
+  wxStaticText* itemStaticText50a = new wxStaticText( itemPanel44, wxID_STATIC, _("Shift double-click action"), wxDefaultPosition, wxDefaultSize, 0 );
+  itemBoxSizer49a->Add(itemStaticText50a, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+
+  m_SDCACB = new wxComboBox( itemPanel44, ID_COMBOBOX, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_DCACBStrings, wxCB_READONLY );
+  itemBoxSizer49a->Add(m_SDCACB, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+
   wxStaticBox* itemStaticBoxSizer52Static = new wxStaticBox(itemPanel44, wxID_ANY, _("Autotype"));
   wxStaticBoxSizer* itemStaticBoxSizer52 = new wxStaticBoxSizer(itemStaticBoxSizer52Static, wxVERTICAL);
   itemBoxSizer45->Add(itemStaticBoxSizer52, 0, wxGROW|wxALL, 5);
@@ -573,6 +568,10 @@ void COptions::CreateControls()
   m_pwpHexCtrl->SetValue(false);
   itemStaticBoxSizer71->Add(m_pwpHexCtrl, 0, wxALIGN_LEFT|wxALL, 5);
 
+  int checkbox_ids[] = {ID_CHECKBOX3, ID_CHECKBOX4, ID_CHECKBOX5, ID_CHECKBOX6, ID_CHECKBOX9};
+  itemPanel70->SetValidator(MultiCheckboxValidator(checkbox_ids, WXSIZEOF(checkbox_ids),
+                            _("At least one type of character (lowercase, uppercase, digits,\nsymbols, hexadecimal) must be permitted."),
+                            _("Password Policy")));
   GetBookCtrl()->AddPage(itemPanel70, _("Password Policy"));
 
   wxPanel* itemPanel102 = new wxPanel( GetBookCtrl(), ID_PANEL4, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
@@ -706,7 +705,7 @@ void COptions::CreateControls()
   itemBoxSizer126->Add(itemCheckBox141, 0, wxALIGN_LEFT|wxALL, 5);
 
 #if defined(__X__) || defined(__WXGTK__)
-  wxCheckBox* itemCheckBox142 = new wxCheckBox( itemPanel125, ID_CHECKBOX35, _("Use Primary Selection for clipboard"), wxDefaultPosition, wxDefaultSize, 0 );
+  wxCheckBox* itemCheckBox142 = new wxCheckBox( itemPanel125, ID_CHECKBOX39, _("Use Primary Selection for clipboard"), wxDefaultPosition, wxDefaultSize, 0 );
   itemCheckBox142->SetValue(false);
   itemBoxSizer126->Add(itemCheckBox142, 0, wxALIGN_LEFT|wxALL, 5);
 #endif
@@ -720,6 +719,23 @@ void COptions::CreateControls()
   itemGrid143->SetColLabelSize(25);
   itemGrid143->SetRowLabelSize(50);
   itemGrid143->CreateGrid(50, 2, wxGrid::wxGridSelectCells);
+  itemGrid143->SetColLabelValue(COL_SHORTCUT_KEY, _("Shortcut Key"));
+  itemGrid143->SetColLabelValue(COL_MENU_ITEM, _("Menu Item"));
+
+  wxBoxSizer* shortcutSizer = new wxBoxSizer(wxVERTICAL);
+  shortcutSizer->Add(itemGrid143, wxSizerFlags().Expand().Proportion(1).Border());
+  wxButton* resetAllShortcuts = new wxButton(itemPanel142, wxID_ANY, _("&Reset All"));
+  shortcutSizer->Add(resetAllShortcuts, wxSizerFlags().Center().Border());
+  itemPanel142->SetSizer(shortcutSizer);
+
+  ShortcutsGridValidator sv(*PWSMenuShortcuts::GetShortcutsManager());
+  itemGrid143->SetValidator(sv);
+  itemGrid143->AutoSize();
+
+  wxGridCellAttr* colAttr = new wxGridCellAttr;
+  colAttr->SetReadOnly();
+  itemGrid143->SetColAttr(COL_MENU_ITEM, colAttr);
+  PWSMenuShortcuts::GetShortcutsManager()->SetShorcutsGridEventHandlers(itemGrid143, resetAllShortcuts);
 
   GetBookCtrl()->AddPage(itemPanel142, _("Shortcuts"));
 
@@ -747,7 +763,7 @@ void COptions::CreateControls()
   itemCheckBox116->SetValidator( wxGenericValidator(& m_secclrclponmin) );
   itemCheckBox117->SetValidator( wxGenericValidator(& m_secclrclponexit) );
   itemCheckBox118->SetValidator( wxGenericValidator(& m_secconfrmcpy) );
-  itemCheckBox119->SetValidator( wxGenericValidator(& m_seclockonwinlock) );
+  itemCheckBox119->SetValidator( wxGenericValidator(& m_seclockonmin) );
   itemCheckBox120->SetValidator( wxGenericValidator(& m_seclockonwinlock) );
   itemCheckBox133->SetValidator( wxGenericValidator(& m_sysstartup) );
   itemSpinCtrl137->SetValidator( wxGenericValidator(& m_sysmaxmru) );
@@ -839,12 +855,15 @@ void COptions::PrefsToPropSheet()
   m_maintaindatetimestamps = prefs->GetPref(PWSprefs::MaintainDateTimeStamps);
   m_escexits = prefs->GetPref(PWSprefs::EscExits);
   m_doubleclickaction = prefs->GetPref(PWSprefs::DoubleClickAction);
-  wxASSERT(m_doubleclickaction >= 0 &&
-           m_doubleclickaction < int(sizeof(DCAStrings)/sizeof(DCAStrings[0])));
   if (m_doubleclickaction < 0 ||
       m_doubleclickaction >= int(sizeof(DCAStrings)/sizeof(DCAStrings[0])))
     m_doubleclickaction = 0;
   m_DCACB->SetValue(DCAStrings[m_doubleclickaction]);
+  m_shiftdoubleclickaction = prefs->GetPref(PWSprefs::ShiftDoubleClickAction);
+  if (m_shiftdoubleclickaction < 0 ||
+      m_shiftdoubleclickaction >= int(sizeof(DCAStrings)/sizeof(DCAStrings[0])))
+    m_shiftdoubleclickaction = 0;
+  m_SDCACB->SetValue(DCAStrings[m_shiftdoubleclickaction]);
   m_minauto = prefs->GetPref(PWSprefs::MinimizeOnAutotype);
   m_autotypeStr = prefs->GetPref(PWSprefs::DefaultAutotypeString).c_str();
   if (m_autotypeStr.empty())
@@ -897,6 +916,16 @@ void COptions::PrefsToPropSheet()
 #endif
 }
 
+static int DCAStr2Int(const wxString &str)
+{
+  for (int i = 0; i < int(sizeof(DCAStrings)/sizeof(DCAStrings[0])); ++i)
+    if (str == DCAStrings[i]) {
+      return i;
+    }
+  ASSERT(0);
+  return -1;
+}
+
 void COptions::PropSheetToPrefs()
 {
   PWSprefs *prefs = PWSprefs::GetInstance();
@@ -937,14 +966,10 @@ void COptions::PropSheetToPrefs()
   // Misc. preferences
   prefs->SetPref(PWSprefs::DeleteQuestion, m_confirmdelete);
   prefs->SetPref(PWSprefs::EscExits, m_escexits);
-  const wxString dcaStr = m_DCACB->GetValue();
-  for (int i = 0; i < int(sizeof(DCAStrings)/sizeof(DCAStrings[0])); ++i)
-    if (dcaStr == DCAStrings[i]) {
-      m_doubleclickaction = i;
-      break;
-    }
-
+  m_doubleclickaction = DCAStr2Int(m_DCACB->GetValue());
   prefs->SetPref(PWSprefs::DoubleClickAction, m_doubleclickaction);
+  m_shiftdoubleclickaction = DCAStr2Int(m_SDCACB->GetValue());
+  prefs->SetPref(PWSprefs::ShiftDoubleClickAction, m_shiftdoubleclickaction);
   prefs->SetPref(PWSprefs::MinimizeOnAutotype, m_minauto);
   prefs->SetPref(PWSprefs::QuerySetDef, m_querysetdef);
   prefs->SetPref(PWSprefs::AltBrowser, tostringx(m_otherbrowser));
@@ -998,19 +1023,21 @@ void COptions::PropSheetToPrefs()
 
   if (m_autotypeStr.empty() || m_autotypeStr == DEFAULT_AUTOTYPE)
       prefs->SetPref(PWSprefs::DefaultAutotypeString, L"", true);
-  else 
+  else
     prefs->SetPref(PWSprefs::DefaultAutotypeString, tostringx(m_autotypeStr), true);
 
+  const bool usehex = m_pwpHexCtrl->GetValue();
   prefs->SetPref(PWSprefs::SavePasswordHistory, m_pwhistsaveCB->GetValue(), true);
   prefs->SetPref(PWSprefs::NumPWHistoryDefault, m_pwhistnumdfltSB->GetValue(), true);
-  prefs->SetPref(PWSprefs::PWDefaultLength, m_pwdefaultlength, true);
-  prefs->SetPref(PWSprefs::PWUseLowercase, m_pwpUseLowerCtrl->GetValue(), true);
-  prefs->SetPref(PWSprefs::PWUseUppercase, m_pwpUseUpperCtrl->GetValue(), true);
-  prefs->SetPref(PWSprefs::PWUseDigits, m_pwpUseDigitsCtrl->GetValue(), true);
-  prefs->SetPref(PWSprefs::PWUseSymbols, m_pwpSymCtrl->GetValue(), true);
-  prefs->SetPref(PWSprefs::PWUseHexDigits, m_pwpHexCtrl->GetValue(), true);
-  prefs->SetPref(PWSprefs::PWUseEasyVision, m_pwpEasyCtrl->GetValue(), true);
-  prefs->SetPref(PWSprefs::PWMakePronounceable, m_pwpPronounceCtrl->GetValue(), true);
+  int pwlenRequired = GetRequiredPWLength();
+  prefs->SetPref(PWSprefs::PWDefaultLength, wxMax(pwlenRequired, m_pwdefaultlength), true);
+  prefs->SetPref(PWSprefs::PWUseLowercase, m_pwpUseLowerCtrl->GetValue() && !usehex, true);
+  prefs->SetPref(PWSprefs::PWUseUppercase, m_pwpUseUpperCtrl->GetValue() && !usehex, true);
+  prefs->SetPref(PWSprefs::PWUseDigits, m_pwpUseDigitsCtrl->GetValue() && !usehex, true);
+  prefs->SetPref(PWSprefs::PWUseSymbols, m_pwpSymCtrl->GetValue() && !usehex, true);
+  prefs->SetPref(PWSprefs::PWUseHexDigits, usehex, true);
+  prefs->SetPref(PWSprefs::PWUseEasyVision, m_pwpEasyCtrl->GetValue() && !usehex, true);
+  prefs->SetPref(PWSprefs::PWMakePronounceable, m_pwpPronounceCtrl->GetValue() && !usehex, true);
   prefs->SetPref(PWSprefs::PWLowercaseMinLength, m_pwpLCSpin->GetValue(), true);
   prefs->SetPref(PWSprefs::PWUppercaseMinLength, m_pwpUCSpin->GetValue(), true);
   prefs->SetPref(PWSprefs::PWDigitMinLength, m_pwpDigSpin->GetValue(), true);
@@ -1024,8 +1051,11 @@ void COptions::OnOk(wxCommandEvent& /* evt */)
 {
   if (Validate() && TransferDataFromWindow()) {
     PropSheetToPrefs();
+    if (PWSMenuShortcuts::GetShortcutsManager()->IsDirty()) {
+      PWSMenuShortcuts::GetShortcutsManager()->ApplyEditedShortcuts();
+    }
+    EndModal(wxID_OK);
   }
-  EndModal(wxID_OK);
 }
 
 
@@ -1051,10 +1081,7 @@ void COptions::OnBackupB4SaveClick( wxCommandEvent& /* evt */ )
 
 void COptions::OnBuPrefix( wxCommandEvent& evt )
 {
-////@begin wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON4 in COptions.
-  // Before editing this code, remove the block markers.
   evt.Skip();
-////@end wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON4 in COptions. 
 }
 
 
@@ -1084,13 +1111,13 @@ void COptions::OnSuffixCBSet( wxCommandEvent& /* evt */ )
   m_bumaxinc->Enable(suffixIndex == INC_SFX);
   switch (suffixIndex) {
   case NO_SFX:
-    m_suffixExample->SetLabel(_(""));
+    m_suffixExample->SetLabel(wxEmptyString);
     break;
   case TS_SFX: {
     time_t now;
     time(&now);
     wxString datetime = PWSUtil::ConvertToDateTimeString(now,
-                                                         TMC_EXPORT_IMPORT).c_str();
+                                                         PWSUtil::TMC_EXPORT_IMPORT).c_str();
       example += L"_";
       example = example + datetime.Left(4) +  // YYYY
         datetime.Mid(5,2) +  // MM
@@ -1225,6 +1252,8 @@ void COptions::OnPwPolUseClick( wxCommandEvent& evt )
   m_pwNumDigbox->Show(showAtLeasts);
   m_pwNumSymbox->Show(showAtLeasts);
 
+  m_pwMinsGSzr->Layout();
+
   m_pwpUseLowerCtrl->Enable(!useHex);
   m_pwpUseUpperCtrl->Enable(!useHex);
   m_pwpUseDigitsCtrl->Enable(!useHex);
@@ -1263,10 +1292,7 @@ void COptions::OnPWHistApply( wxCommandEvent& evt )
 {
   // XXX TBD - send this to someone who knows how to deal with it!
 
-////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PWHISTNOCHANGE in COptions.
-  // Before editing this code, remove the block markers.
   evt.Skip();
-////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PWHISTNOCHANGE in COptions. 
 }
 
 
@@ -1300,3 +1326,46 @@ void COptions::OnUseSystrayClick( wxCommandEvent& /* evt */)
   m_sysmaxREitemsSB->Enable(m_sysusesystrayCB->GetValue());
 }
 
+void COptions::OnPageChanging(wxBookCtrlEvent& evt)
+{
+  const int from = evt.GetOldSelection();
+  if (from != -1) {
+    wxWindow* page = GetBookCtrl()->GetPage(from);
+    //note that wxWindow::Validate() validates child windows
+    //we need to validate the page itself, so we call its Validator directly
+    wxValidator* validator = page->GetValidator();
+    if (validator && !validator->Validate(this))
+      evt.Veto();
+  }
+}
+
+/*
+ * Just trying to give the user some visual indication that
+ * the password length has to be bigger than the sum of all
+ * "at least" lengths.  This is not comprehensive & foolproof
+ * since there are far too many ways to make the password length
+ * smaller than the sum of "at least" lengths, to even think of.
+ *
+ * In OnOk(), we just ensure the password length is greater than
+ * the sum of all enabled "at least" lengths.  We have to do this in the
+ * UI, or else password generation crashes
+ */
+void COptions::OnAtLeastChars(wxSpinEvent& /*evt*/)
+{
+  const int min = GetRequiredPWLength();
+  wxSpinCtrl* pwlenCtrl = wxDynamicCast(FindWindow(ID_SPINCTRL3), wxSpinCtrl);
+  //pwlenCtrl->SetRange(min, pwlenCtrl->GetMax());
+  if (min > pwlenCtrl->GetValue())
+    pwlenCtrl->SetValue(min);
+}
+
+int COptions::GetRequiredPWLength() const {
+  wxSpinCtrl* spinCtrls[] = {m_pwpUCSpin, m_pwpLCSpin, m_pwpDigSpin, m_pwpSymSpin};
+  int total = 0;
+  for (size_t idx = 0; idx < WXSIZEOF(spinCtrls); ++idx) {
+    if (spinCtrls[idx]->IsEnabled())
+      total += spinCtrls[idx]->GetValue();
+  }
+  return total;
+}
+
