@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -7,7 +7,7 @@
  */
 
 /** \file pwsafeapp.cpp
-* 
+*
 */
 
 // For compilers that support precompilation, includes "wx/wx.h".
@@ -51,6 +51,8 @@ using namespace std;
 #include <wx/clipbrd.h>
 #endif
 #include <wx/snglinst.h>
+#include "../../core/PWSLog.h"
+#include "./pwsmenushortcuts.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/msvcrt.h>
@@ -138,19 +140,76 @@ BEGIN_EVENT_TABLE( PwsafeApp, wxApp )
 
 ////@begin PwsafeApp event table entries
 ////@end PwsafeApp event table entries
-    EVT_ACTIVATE_APP(PwsafeApp::OnActivate)
-    EVT_TIMER(ACTIVITY_TIMER_ID, PwsafeApp::OnActivityTimer)
-    EVT_CUSTOM(wxEVT_GUI_DB_PREFS_CHANGE, wxID_ANY, PwsafeApp::OnDBGUIPrefsChange)
+EVT_ACTIVATE_APP(PwsafeApp::OnActivate)
+EVT_TIMER(ACTIVITY_TIMER_ID, PwsafeApp::OnActivityTimer)
+EVT_CUSTOM(wxEVT_GUI_DB_PREFS_CHANGE, wxID_ANY, PwsafeApp::OnDBGUIPrefsChange)
 END_EVENT_TABLE()
 
 
+
+void PwsafeApp::initLanguageSupport()
+{
+  long language = wxLocale::GetSystemLanguage();
+#if defined(__UNIX__) && !defined(__WXMAC__)
+  // Workaround for wx bug 15006
+  if (language == wxLANGUAGE_UNKNOWN) {
+    std::wcerr << L"Couldn't detect locale. Trying to skip empty env. variables."
+		 << endl;
+    // Undefine empty environment variables and try again
+    wxString langFull;
+    if (wxGetEnv(wxT("LC_ALL"), &langFull) && !langFull){
+      wxUnsetEnv(wxT("LC_ALL"));
+      std::wcerr << L"Empty LC_ALL variable was dropped" << endl;
+    }
+    if (wxGetEnv(wxT("LC_MESSAGES"), &langFull) && !langFull){
+      wxUnsetEnv(wxT("LC_MESSAGES"));
+      std::wcerr << L"Empty LC_MESSAGES variable was dropped" << endl;
+    }
+    if (wxGetEnv(wxT("LANG"), &langFull) && !langFull){
+      wxUnsetEnv(wxT("LANG"));
+      std::wcerr << L"Empty LANG variable was dropped" << endl;
+    }
+    language = wxLocale::GetSystemLanguage();
+  }
+#endif
+  // load language if possible, fall back to english otherwise
+  if(wxLocale::IsAvailable(language)) {
+    m_locale = new wxLocale( language, wxLOCALE_CONV_ENCODING );
+
+    // add locale search paths
+    m_locale->AddCatalogLookupPathPrefix(wxT("/usr"));
+    m_locale->AddCatalogLookupPathPrefix(wxT("/usr/local"));
+#if defined(__WXDEBUG__) || defined(_DEBUG) || defined(DEBUG)
+    m_locale->AddCatalogLookupPathPrefix(wxT("../I18N/mos"));
+#endif
+    if (!m_locale->AddCatalog(wxT("pwsafe"))) {
+      std::wcerr << L"Couldn't load text for "
+		 << m_locale->GetLanguageName(language).c_str() << endl;
+    }
+
+    if(! m_locale->IsOk()) {
+      std::cerr << "selected language is wrong" << std::endl;
+      delete m_locale;
+      m_locale = new wxLocale( wxLANGUAGE_ENGLISH );
+      language = wxLANGUAGE_ENGLISH;
+    }
+  } else {
+    std::cerr << "The selected language is not supported by your system."
+	      << "Try installing support for this language." << std::endl;
+    m_locale = new wxLocale( wxLANGUAGE_ENGLISH );
+    language = wxLANGUAGE_ENGLISH;
+  }
+  // add translation for standard resources
+  m_locale->AddCatalog(wxT("wxstd"));
+}
+
 /*!
  * Constructor for PwsafeApp
  */
 
 PwsafeApp::PwsafeApp() : m_activityTimer(new wxTimer(this, ACTIVITY_TIMER_ID)),
-  m_frame(0), m_recentDatabases(0),
-  m_controller(new wxHtmlHelpController)
+			 m_frame(0), m_recentDatabases(0),
+			 m_controller(new wxHtmlHelpController), m_locale(NULL)
 {
     Init();
 }
@@ -165,8 +224,10 @@ PwsafeApp::~PwsafeApp()
 
   PWSprefs::DeleteInstance();
   PWSrand::DeleteInstance();
-  
+  PWSLog::DeleteLog();
+
   delete m_controller;
+  delete m_locale;
 }
 
 /*!
@@ -175,6 +236,7 @@ PwsafeApp::~PwsafeApp()
 
 void PwsafeApp::Init()
 {
+  initLanguageSupport();
 ////@begin PwsafeApp member initialisation
 ////@end PwsafeApp member initialisation
 }
@@ -184,7 +246,7 @@ void PwsafeApp::Init()
  */
 
 bool PwsafeApp::OnInit()
-{    
+{
   SetAppName(pwsafeAppName);
   m_core.SetApplicationNameAndVersion(tostdstring(pwsafeAppName),
                                       DWORD((MINORVERSION << 16) | MAJORVERSION));
@@ -217,7 +279,6 @@ bool PwsafeApp::OnInit()
   wxString filename, user, host, cfg_file;
   bool cmd_ro = cmdParser.Found(wxT("r"));
   // Next variable currently not referenced
-  bool cmd_validate = cmdParser.Found(wxT("v"), &filename);
   bool cmd_encrypt = cmdParser.Found(wxT("e"), &filename);
   bool cmd_decrypt = cmdParser.Found(wxT("d"), &filename);
   bool cmd_closed = cmdParser.Found(wxT("c"));
@@ -275,10 +336,10 @@ bool PwsafeApp::OnInit()
   }
 #endif /* _DEBUG */
 
-  wxSingleInstanceChecker appInstance;
-  if (!prefs->GetPref(PWSprefs::MultipleInstances) && 
+  static wxSingleInstanceChecker appInstance;
+  if (!prefs->GetPref(PWSprefs::MultipleInstances) &&
         (appInstance.Create(wxT("pwsafe.lck"), towxstring(pws_os::getuserprefsdir())) &&
-         appInstance.IsAnotherRunning())) 
+         appInstance.IsAnotherRunning()))
   {
     wxMessageBox(_("Another instance of Password Safe is already running"), _("Password Safe"),
                           wxOK|wxICON_INFORMATION);
@@ -302,11 +363,10 @@ bool PwsafeApp::OnInit()
     //dbox.SetStartClosed(true);
     // dbox.SetStartSilent(true);
   }
-  // dbox.SetValidate(cmd_validate);
 
   //Initialize help subsystem
   wxFileSystem::AddHandler(new wxArchiveFSHandler);
-  
+
   wxString helpfile(wxFileName(towxstring(pws_os::gethelpdir()), wxT("help.zip")).GetFullPath());
   if (!m_controller->Initialize(helpfile))
     wxMessageBox(_("Could not initialize help subsystem. Help will not be available"),
@@ -330,9 +390,7 @@ bool PwsafeApp::OnInit()
     wxASSERT_MSG(!m_frame, wxT("Frame window created unexpectedly"));
     m_frame = new PasswordSafeFrame(NULL, m_core);
     m_frame->Load(initWindow->GetPassword());
-    if (cmd_validate)
-      m_frame->ValidateCurrentDatabase();
-  } 
+  }
   else {
     wxASSERT_MSG(!m_frame, wxT("Frame window created unexpectedly"));
     m_frame = new PasswordSafeFrame(NULL, m_core);
@@ -340,23 +398,44 @@ bool PwsafeApp::OnInit()
 
   RestoreFrameCoords();
   m_frame->Show();
+  SetTopWindow(m_frame);
   return true;
 }
 
+void PwsafeApp::StopIdleTimer()
+{
+  m_activityTimer->Stop();
+}
+
+void PwsafeApp::StartIdleTimer()
+{
+  int timeout = PWSprefs::GetInstance()->GetPref(PWSprefs::IdleTimeout);
+  if (timeout != 0)
+    m_activityTimer->Start(timeout*60*1000, true);
+}
+
+bool PwsafeApp::IsIdleTimerRunning() const
+{
+  return m_activityTimer->IsRunning();
+}
 
 /*!
  * Cleanup for PwsafeApp
  */
 
 int PwsafeApp::OnExit()
-{    
+{
+  StopIdleTimer();
   recentDatabases().Save();
   PWSprefs *prefs = PWSprefs::GetInstance();
   if (!m_core.GetCurFile().empty())
     prefs->SetPref(PWSprefs::CurrentFile, m_core.GetCurFile());
   // Save Application related preferences
   prefs->SaveApplicationPreferences();
-  m_activityTimer->Stop();
+  // Save shortcuts, if changed
+  PWSMenuShortcuts::GetShortcutsManager()->SaveUserShortcuts();
+
+  PWSMenuShortcuts::DestroyShortcutsManager();
 ////@begin PwsafeApp cleanup
   return wxApp::OnExit();
 ////@end PwsafeApp cleanup
@@ -364,28 +443,28 @@ int PwsafeApp::OnExit()
 
 void PwsafeApp::OnActivate(wxActivateEvent& actEvent)
 {
-  if (actEvent.GetActive()) {
-    m_activityTimer->Stop();
-  }
-  else {
-    m_activityTimer->Stop();
-    m_activityTimer->Start(PWSprefs::GetInstance()->GetPref(PWSprefs::IdleTimeout)*60*1000, true);
+  StopIdleTimer();
+  if (!actEvent.GetActive()) {
+    StartIdleTimer();
   }
   actEvent.Skip();
 }
 
-void PwsafeApp::OnActivityTimer(wxTimerEvent& /* timerEvent */)
+void PwsafeApp::OnActivityTimer(wxTimerEvent &evt)
 {
-  if (!m_frame->GetCurrentSafe().IsEmpty())
-    m_frame->HideUI(true);  //true => lock
+  if (evt.GetId() == ACTIVITY_TIMER_ID && PWSprefs::GetInstance()->GetPref(PWSprefs::LockDBOnIdleTimeout)) {
+    if (m_frame != NULL && !m_frame->GetCurrentSafe().IsEmpty())
+      m_frame->HideUI(true);  //true => lock
+  }
 }
 
 void PwsafeApp::OnDBGUIPrefsChange(wxEvent& evt)
 {
   UNREFERENCED_PARAMETER(evt);
-  if (m_activityTimer->IsRunning()) {
-    m_activityTimer->Stop();
-    m_activityTimer->Start(PWSprefs::GetInstance()->GetPref(PWSprefs::IdleTimeout)*60*1000, true);
+  if (IsIdleTimerRunning()) {
+    // Restart, in case Idle timer settings have just changed
+    StopIdleTimer();
+    StartIdleTimer();
   }
 }
 
@@ -405,9 +484,9 @@ void PwsafeApp::SaveFrameCoords(void)
     PWSprefs::GetInstance()->SetPrefRect(-1, -1, -1, -1);
   }
   else if (m_frame->IsIconized() || !m_frame->IsShown()) {
-    //if we save coords when minimized/hidden, pwsafe could be hidden or off 
+    //if we save coords when minimized/hidden, pwsafe could be hidden or off
     //screen next time it runs
-    return; 
+    return;
   }
   else {
     wxRect rc = m_frame->GetScreenRect();
@@ -421,19 +500,19 @@ void PwsafeApp::RestoreFrameCoords(void)
   PWSprefs::GetInstance()->GetPrefRect(top, bottom, left, right);
   if (!(left == -1 && top == -1 && right == -1 && bottom == -1)) {
     wxRect rcApp(left, top, right - left, bottom - top);
-    
+
     int displayWidth, displayHeight;
     ::wxDisplaySize(&displayWidth, &displayHeight);
     wxRect rcDisplay(0, 0, displayWidth, displayHeight);
-    
+
     if (!rcApp.IsEmpty() && rcDisplay.Contains(rcApp))
       m_frame->SetSize(rcApp);
   }
 }
 
 int PwsafeApp::FilterEvent(wxEvent& evt) {
-  if (evt.IsCommandEvent() && evt.GetId() == wxID_HELP && 
-          (evt.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED || 
+  if (evt.IsCommandEvent() && evt.GetId() == wxID_HELP &&
+          (evt.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED ||
             evt.GetEventType() == wxEVT_COMMAND_MENU_SELECTED)) {
     OnHelp(*wxDynamicCast(&evt, wxCommandEvent));
     return int(true);
@@ -449,7 +528,7 @@ void PwsafeApp::OnHelp(wxCommandEvent& evt)
     //we can't get to its parent
     if (win->GetId() == wxID_HELP && ((win = win->GetParent()) == NULL))
       return;
-    
+
     wxString keyName, msg;
     //Is this a property sheet?
     wxPropertySheetDialog* propSheet = wxDynamicCast(win, wxPropertySheetDialog);
@@ -457,27 +536,27 @@ void PwsafeApp::OnHelp(wxCommandEvent& evt)
       const wxString dlgName = win->GetClassInfo()->GetClassName();
       const wxString pageName = propSheet->GetBookCtrl()->GetPageText(propSheet->GetBookCtrl()->GetSelection());
       keyName = dlgName + wxT('#') + pageName;
-      msg << wxT("Missing help definition for page \"") << pageName
-          << wxT("\" of \"") << dlgName 
+      msg << _("Missing help definition for page \"") << pageName
+          << _("\" of \"") << dlgName
           << wxT("\".\n");
     } else { // !propSheet
       keyName = win->GetClassInfo()->GetClassName();
-      msg << wxT("Missing help definition for window \"") << keyName
+      msg << _("Missing help definition for window \"") << keyName
           << wxT("\".\n");
     }
-    
+
     StringToStringMap& helpmap = GetHelpMap();
     StringToStringMap::iterator itr = helpmap.find(keyName);
     if (itr != helpmap.end())
       m_controller->DisplaySection(itr->second);
     else {
 #ifdef __WXDEBUG__
-      msg << wxT("Please inform the developers.");
+      msg << _("Please inform the developers.");
       wxMessageBox(msg, _("Help Undefined"), wxOK | wxICON_EXCLAMATION);
 #endif
     } // keyName not found in map
   } else {
-    //just display the main page.  Could happen if the click came from a menu instead of 
+    //just display the main page.  Could happen if the click came from a menu instead of
     //a button, like for the top-level frame
     m_controller->DisplayContents();
   }
@@ -489,7 +568,7 @@ PwsafeApp::StringToStringMap& PwsafeApp::GetHelpMap()
   //I don't want to introduce yet another set of source/header files just for this
   static StringToStringMap helpMap;
   static bool initialized = false;
-  
+
   if (!initialized) {
 #define DLG_HELP(dlgname, htmlfile) helpMap[wxSTRINGIZE_T(dlgname)] = wxSTRINGIZE_T(htmlfile);
 #define PROPSHEET_HELP(sheet, page, htmlfile) helpMap[wxString(wxSTRINGIZE_T(sheet) wxT("#")) + page] = wxSTRINGIZE_T(htmlfile);
